package cn.wisenergy.model.dto;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.List;

/**
 *
 */
@Data
@ApiModel
public class CameraPlaybacksURLs implements Serializable {

    /**
     * 监控点唯一标识，分页获取监控点资源接口获取返回参数cameraIndexCode
     */
    @NotBlank
    @ApiModelProperty(required = true,value = "监控点唯一标识")
    private List<String> cameraIndexCodes;

    /**
     * 存储类型,0：中心存储
     * 1：设备存储
     * 默认为中心存储
     */
    @ApiModelProperty(value = "存储类型,0：中心存储,1：设备存储,默认为中心存储")
    private Integer recordLocation;

    /**
     * 取流协议（应用层协议)，
     * “hik”:HIK私有协议，使用视频SDK进行播放时，传入此类型；
     * “rtsp”:RTSP协议；
     * “rtmp”:RTMP协议（RTMP回放要求录像片段连续，需要在URL后自行拼接beginTime=20190902T100303&endTime=20190902T100400，其中20190902T100303至20190902T100400为查询出有连续录像的时间段。对于不连续的录像，需要分段查询分段播放）；
     * “hls”:HLS协议（HLS协议只支持海康SDK协议、EHOME协议、ONVIF协议接入的设备；只支持H264视频编码和AAC音频编码；hls协议只支持云存储，不支持设备存储，云存储版本要求v2.2.4及以上的2.x版本，或v3.0.5及以上的3.x版本；ISC版本要求v1.2.0版本及以上，需在运管中心-视频联网共享中切换成启动平台外置VOD）。
     * 参数不填，默认为HIK协议
     */
    @ApiModelProperty(value = "取流协议（应用层协议)",notes = "hik、rtsp、rtmp、hls")
    private String protocol;

    /**
     * 开始查询时间（IOS8601格式：yyyy-MM-dd’T’HH:mm:ss.SSSXXX）
     * 例如北京时间：
     * 2017-06-14T00:00:00.000+08:00
     */
    @NotBlank
    @ApiModelProperty(required = true,value = "开始查询时间（IOS8601格式：yyyy-MM-dd’T’HH:mm:ss.SSSXXX）")
    private String beginTime;

    /**
     * 结束查询时间，开始时间和结束时间相差不超过三天；
     * （IOS8601格式：yyyy-MM-dd’T’HH:mm:ss.SSSXXX）例如北京时间：
     * 2017-06-15T00:00:00.000+08:00
     */
    @NotBlank
    @ApiModelProperty(required = true,value = "结束查询时间，开始时间和结束时间相差不超过三天；（IOS8601格式：yyyy-MM-dd’T’HH:mm:ss.SSSXXX）例如北京时间：")
    private String endTime;

    /**
     * 分页查询id，上一次查询返回的uuid，用于继续查询剩余片段，默认为空字符串。
     * 当存储类型为设备存储时，该字段生效，中心存储会一次性返回全部片段。
     */
    @ApiModelProperty(value = "分页查询id，上一次查询返回的uuid，用于继续查询剩余片段，默认为空字符串。")
    private String uuid;

    /**
     * 查询录像的锁定类型，0-查询全部录像；1-查询未锁定录像；2-查询已锁定录像，不传默认值为0
     */
    @ApiModelProperty(value = "查询录像的锁定类型，0-查询全部录像；1-查询未锁定录像；2-查询已锁定录像，不传默认值为0")
    private Integer lockType;
}
