package cn.wisenergy.model.dto;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import java.io.Serializable;

/**
 *
 */
@Data
public class CameraRecordLock implements Serializable {

    /**
     * 监控点唯一标识，分页获取监控点资源接口获取返回参数cameraIndexCode
     */
    @NotBlank
    @ApiModelProperty(value = "监控点唯一标识", required = true)
    private String cameraIndexCode;

    /**
     * 下发的存储类型。0-中心存储，1-设备存储。如果不传入，默认为0-中心存储
     */
    @ApiModelProperty(value = "下发的存储类型。0-中心存储，1-设备存储。如果不传入，默认为0-中心存储")
    private Integer type;

    /**
     * 开始时间（IOS8601格式：yyyy-MM-dd’T’HH:mm:ss.SSSzzz +当前时区，例如北京时间：2018-07-26T15:00:00.000+08:00）
     */
    @NotBlank
    @ApiModelProperty(required = true, value = "开始时间（IOS8601格式：yyyy-MM-dd’T’HH:mm:ss.SSSzzz +当前时区，")
    private String startTime;

    /**
     * 结束时间（IOS8601格式：yyyy-MM-dd’T’HH:mm:ss.SSSzzz +当前时区，例如北京时间：2018-07-26T15:00:00.000+08:00）
     */
    @NotBlank
    @ApiModelProperty(required = true, value = "结束时间（IOS8601格式：yyyy-MM-dd’T’HH:mm:ss.SSSzzz +当前时区，")
    private String endTime;

    /**
     * 锁定时长，单位：秒，为0时表示解锁，取值范围：0-1576800000
     */
    @Min(value = 0)
    @ApiModelProperty(value = "锁定时长，单位：秒，为0时表示解锁，取值范围：0-1576800000")
    private Integer lockTime;
}
