package cn.wisenergy.model.dto;

import lombok.Data;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import java.io.Serializable;

/**
 *
 */
@Data
public class CameraRegionNodes implements Serializable {

    /**
     * 查询资源的类型，传region时查询的为用户有配置权限的区域树，
     * 传资源类型如：camera、encodeDevice查询用户对该类资源有权限的区域树；
     */
    @NotBlank
    private String resourceType;
    /**
     * 父编号集合;
     * 父编号个数
     * <=1000个；
     */
    private String[] parentIndexCodes;
    /**
     * 是否包含下级区域
     * true: 搜索parentIndexCodes的所有子、孙区域；
     * false: 只搜索parentIndexCodes的直接子区域；
     */
    private boolean isSubRegion;

    /**
     * 当前页码；pageNo≥1
     */
    @Min(value = 1)
    private Integer pageNo;
    /**
     * 分页大小；0<pageSize≤1000
     */
    @Max(value = 1000)
    private Integer pageSize;
    /**
     * 区域类型
     * 10: 普通区域
     * 11: 级联区域
     * 12: 楼栋单元
     */
    private Integer regionType;
    /**
     * 区域名称；根据区域名称过滤，可模糊查询；
     */
    private String regionName;
    /**
     * 级联标识，默认0
     * 0: 全部，
     * 1: 本级，
     * 2: 级联
     */
    private Integer cascadeFlag;


}
