package cn.wisenergy.service;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.dto.*;

public interface ICameraApiService {

    /**
     * 获取监控点预览取流URLv2
     * 综合安防管理平台iSecure Center V1.4及以上版本
     *
     * @param param
     * @return
     */
    R<Object> camerasPreviewUrls(CameraPreviewUrls param);


    /**
     * 分页获取监控点资源
     *
     * @param param
     * @return
     */
    R<Object> cameras(CameraPage param);


    /**
     * 获取监控点回放取流URLv2
     *
     * @param param
     * @return
     */
    R playbackURLs(CameraPlaybackURLs param);

    /**
     * 语音对讲  查询对讲URL
     *
     * @param param
     * @return
     */
    R camerasTalkURLs(CameraTalkURLs param);

    /**
     * 抓图 手动抓图
     *
     * @param cameraIndexCode 监控点唯一标识，可通过分页获取监控点资源获取
     * @return
     */
    R manualCapture(CameraIndexCode cameraIndexCode);

    /**
     * 手动录像 录像锁定与解锁
     *
     * @return
     */
    R recordLock(CameraRecordLock param);

    /**
     * 手动录像 开始手动录像
     * 1.该接口用于向存储接入服务下发开始手动录像请求，将前端实时流存储到存储设备上，
     * 支持中心存储和设备存储两种存储模式。萤石设备不支持手动录像功能。
     *
     * @return
     */
    R manualRecordStart(CameraManualRecordStart param);


    /**
     * 手动录像 停止手动录像
     * 该接口用于根据手动录像任务ID，向存储接入服务下发停止手动录像请求, 支持中心存储和设备存储两种存储模式。
     * 萤石设备不支持手动录像功能。
     * 综合安防管理平台iSecure Center V1.5及以上版本
     *
     * @return
     */
    R manualRecordStop(CameraManualRecordStop param);

    /**
     * 手动录像 获取手动录像状态
     * 该接口用于根据手动录像任务ID，向存储接入服务下发查询手动录像状态请求, 设备存储暂不支持该能力。
     * 萤石设备不支持手动录像功能。
     * 综合安防管理平台iSecure Center V1.5及以上版本
     *
     * @return
     */
    R manualRecordStatus(CameraManualRecordStop param);

    /**
     * 手动录像 查询手动录像编号
     * 该接口用于根据用户id获取当前用户创建的手动录像任务ID，用于获取手动录像状态和停止手动录像。
     * 综合安防管理平台iSecure Center V1.4及以上版本
     *
     * @return
     */
    R manualRecordTaskIdSearch(CameraIndexCodes cameraIndexCodes);


    /**
     * 根据区域编号获取下一级区域列表v2，下级区域只包括直接下级子区域。
     *
     * @return
     */
    R regionsSubRegions(CameraRegionsSubRegions regionsSubRegions);

    /**
     * 获取根区域信息
     *
     * @return
     */
    R regionsRoot();

    /**
     * 查询区域列表v2
     *
     * @param regionNodes
     * @return
     */
    R regionNodesByParams(CameraRegionNodes regionNodes);

    /**
     * 根据区域编号获取下级监控点列表
     *
     * @param param
     * @return
     */
    R regionIndexCodeCameras(CamerasRegionIndexCode param);

    /**
     * 获取监控点在线状态
     *
     * @return
     */
    R onlineCamera(CameraOnlineStatus status);

    /**
     *获取编码设备在线状态
     * @return
     */
    R onlineEncodeDevice(EncodeDeviceOnlineStatus status);
}
