package cn.wisenergy.service.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.vo.*;

import java.util.List;

/**
 * @ Description   :  社区大脑接口定义
 * @ Author        :  lcc
 * @ CreateDate    :  2020/12/8 18:02
 */
public interface CommunityBrainService {
    /**
     * 根据登录用户获取统计详情
     *
     * @param autoId 用户autoId
     * @return 统计详情
     */
    R<CountDetailVo> getCountDetail(String autoId);

    /**
     * 社区大脑-人员总数
     *
     * @param autoId 用户autoId
     * @return 人员总数
     */
    R<List<UserRoleCountVo>> userRoleCount(String autoId);

    /**
     * 社区大脑 小区/街道督查率排名
     *
     * @param autoId 用户autoId
     * @param type   1: 街道 2：社区
     * @return 小区/街道督查率排名
     */
    R<List<InspectorRateVo>> getInspectorRate(String autoId, Integer type);


    /**
     * 违规派单
     *
     * @param autoId 用户autoId
     * @return 违规派单列表
     */
    R<List<ViolationOrderVo>> getViolationOrder(String autoId);

    /**
     * 社区大脑-总派单数/派单反馈数
     *
     * @param autoId 用户autoId
     * @param type   1:今日 2：本周 3：本月 4：本季 5：本年
     * @return 总派单数/派单反馈数
     */
    R<SendOrderCountVo> getSendOrderCount(String autoId, Integer type);

    /**
     * 社区大脑-清运团队清运率/清运次数统计
     *
     * @param autoId 用户autoId
     * @return 清运团队清运率/清运次数统计
     */
    R<List<ClearTeamVo>> getClearTeam(String autoId);
}
