package cn.wisenergy.service.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.dto.*;
import cn.wisenergy.model.vo.CameraIndexVo;

import java.util.List;

/**
 * 站点值守
 */
public interface ISiteWatchOverService {

    /**
     * 查询视频监控点
     *
     * @param qry
     * @return
     */
    R pageVideo(VideoPageQry qry);

    List<CameraMonitory> getCameraMonitory(VideoPageQry qry);

    /**
     * 根据监控点查询视频预览URL
     *
     * @param previewUrls
     * @return
     */
    R videoUrls(CamerasPreviewUrls previewUrls);

    /**
     * 根据一个监控点查询视频预览Urls
     *
     * @param previewUrl
     * @return
     */
    R videoUrl(CameraPreviewUrls previewUrl);

    /**
     * 根据一个监控点查询录像回放Urls
     *
     * @param playbackURL
     * @return
     */
    R playbackUrl(CameraPlaybackURLs playbackURL);

    /**
     * 根据多个监控点查询录像回放Urls
     *
     * @param playbackURLs
     * @return
     */
    R playbackUrls(CameraPlaybacksURLs playbackURLs);

    /**
     * 根据一个监控点进行手动抓图
     *
     * @param cameraIndexCode
     * @return
     */
    R manualCapture(CameraIndexCode cameraIndexCode);

    /**
     * 根据一个监控点进行录像锁定与解锁
     *
     * @param recordLock
     * @return
     */
    R recordLock(CameraRecordLock recordLock);

    /**
     * 根据一个监控点进行开始录像操作
     *
     * @param recordStart
     * @return
     */
    R recordStart(CameraManualRecordStart recordStart);

    /**
     * 根据一个监控点进行手动录像停止操作
     *
     * @param recordStop
     * @return
     */
    R recordStop(CameraManualRecordStop recordStop);

    /**
     * 根据一个监控点进行手动录像状态获取操作
     *
     * @param recordStatus
     * @return
     */
    R recordStatus(CameraManualRecordStop recordStatus);

    /**
     * @param recordSearch
     * @return
     */
    R recordSearch(CameraIndexCodes recordSearch);

    /**
     * 根据条件分页参数获取监控点在线状态列表
     *
     * @param onlineStatus
     * @return
     */
    R onlineStatus(CameraOnlineStatus onlineStatus);

    /**
     * 分页获取安防平台监控点列表
     *
     * @param cameraPage
     * @return
     */
    R pageCameraIndex(CameraPage cameraPage);

    /**
     * 获取垃圾站下摄像头code
     *
     * @param autoId
     * @return
     */
    R<List<CameraIndexVo>> getByAutoId(String autoId, String userAutoId);

    /**
     * 通过社区查询垃圾站下的摄像头回放URLs，带分页
     *
     * @param cameraListDto
     * @return
     */
    R pageCameraBackUrls(CameraListDto cameraListDto);
}
