package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.constant.CommonAttributes;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.AccountOrganizationMapper;
import cn.wisenergy.mapper.CameraMapper;
import cn.wisenergy.mapper.RubbishMapper;
import cn.wisenergy.model.app.Camera;
import cn.wisenergy.model.app.Rubbish;
import cn.wisenergy.model.dto.CameraOnlineStatus;
import cn.wisenergy.model.vo.CameraQueryVo;
import cn.wisenergy.model.vo.OnlineVo;
import cn.wisenergy.model.vo.RubbishNameVo;
import cn.wisenergy.service.app.CameraService;
import cn.wisenergy.service.app.Daosupport;
import cn.wisenergy.service.app.ISiteWatchOverService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;


@Service
@Slf4j
public class CameraManagerServiceImpl extends ServiceImpl<CameraMapper, Camera> implements CameraService {
    @Autowired
    private CameraMapper cameraMapper;

    @Autowired
    private RubbishMapper rubbishMapper;

    @Autowired
    private Daosupport daosupport;

    @Autowired
    private AccountOrganizationMapper organizationMapper;

    @Autowired
    private ISiteWatchOverService siteWatchOverService;

    @Override
    public R<Camera> add(Camera camera) {
        log.info("ljfl-service[]CameraManagerServiceImpl[]add[]input.param.camera:{}" + camera);
        if (null == camera) {
            return R.error("入参为空!");
        }

        //1、检查摄像机参数
        if (StringUtils.isBlank(camera.getCameraNumber())) {
            return R.error("摄像机编号不能为空!");
        }

        if (StringUtils.isBlank(camera.getName())) {
            return R.error("摄像机名称不能为空!");
        }

        if (StringUtils.isBlank(camera.getInspectorName())) {
            return R.error("督察员姓名不能为空!");
        }

        if (StringUtils.isBlank(camera.getInspectorPhone())) {
            return R.error("督察员手机号不能为空!");
        }

        if (null == camera.getRubbishId()) {
            return R.error("所属投放点不能为空!");
        }

        //2、检查摄像机编号是否重复
        //flag为1，代表新增，flag为2代表编辑
        Integer flag = 1;
        Boolean bool = checkCameraNumber(flag, camera);
        if (!bool) {
            return R.error("摄像机编号重复!");
        }

        //获取垃圾点名称
        Rubbish rubbish = rubbishMapper.getById(camera.getRubbishId());
        if (null != rubbish) {
            camera.setRubbishName(rubbish.getRubbishName());
        }
        camera.setStatus(0);
        camera.setIsDelete(0);
        int count = cameraMapper.add(camera);
        if (count == 0) {
            return R.error("添加摄像头失败");
        }
        return R.ok(camera);
    }

    @Override
    public R<Boolean> edit(Camera camera) {
        log.info("ljfl-service[]CameraManagerServiceImpl[]edit[]input.param.camera:{}" + camera);
        if (null == camera) {
            return R.error("入参为空");
        }

        //1、检查摄像机参数
        if (StringUtils.isBlank(camera.getCameraNumber())) {
            return R.error("摄像机编号不能为空!");
        }

        if (StringUtils.isBlank(camera.getName())) {
            return R.error("摄像机名称不能为空!");
        }

        if (StringUtils.isBlank(camera.getInspectorName())) {
            return R.error("督察员姓名不能为空!");
        }

        if (StringUtils.isBlank(camera.getInspectorPhone())) {
            return R.error("督察员手机号不能为空!");
        }

        if (null == camera.getRubbishId()) {
            return R.error("所属投放点不能为空!");
        }

        //2、检查摄像机编号是否重复
        //flag为1，代表新增，flag为2代表编辑
        Integer flag = 2;
        this.checkCameraNumber(flag, camera);

        //2.根据id,获取摄像机信息
        Camera cameraReal = cameraMapper.selectById(camera.getId());
        if (null == cameraReal) {
            return R.error("该摄像机信息不存在");
        }

        //3获取垃圾点名称
        Rubbish rubbish = rubbishMapper.selectById(camera.getRubbishId());
        if (null != rubbish) {
            cameraReal.setRubbishName(rubbish.getRubbishName());
        }

        //4、编辑修改内容
        cameraReal.setInspectorName(camera.getInspectorName());
        cameraReal.setCameraNumber(camera.getCameraNumber());
        cameraReal.setInspectorPhone(camera.getInspectorPhone());
        cameraReal.setLatitude(camera.getLatitude());
        cameraReal.setLongitude(camera.getLongitude());
        cameraReal.setName(camera.getName());
        cameraReal.setRemark(camera.getRemark());
        cameraReal.setRubbishId(camera.getRubbishId());
        int count = cameraMapper.modify(cameraReal);
        if (count == 0) {
            return R.ok(1, false);
        }
        return R.ok(0, true);
    }

    @Override
    public R<Boolean> delete(Integer id) {
        log.info("ljfl-service[]CameraManagerServiceImpl[]delete[]input.param.id:{}" + id);
        if (null == id) {
            return R.error("入参为空");
        }

        //1、根据id,获取摄像机信息
        Camera camera = cameraMapper.selectById(id);
        if (null != camera) {
            camera.setIsDelete(1);
            int count = cameraMapper.modify(camera);
            if (count == 1) {
                return R.ok(0, true);
            }
        }
        return R.ok(1, false);
    }

    @Override
    public R<PageInfo<Camera>> getList(CameraQueryVo cameraQueryVo) {
        log.info("ljfl-service[]CameraManagerServiceImpl[]getList[]input.param.cameraQueryVo:{}" + cameraQueryVo);
        if (null == cameraQueryVo) {
            return R.error("入参为空");
        }
        pageHandle(cameraQueryVo);
        Integer total;
        StringBuilder sql = new StringBuilder();
        sql.append("select count(1) from camera where is_delete=0 and status=0 ");
        if (StringUtils.isNotEmpty(cameraQueryVo.getFieldName()) &&
                StringUtils.isNotEmpty(cameraQueryVo.getCondition()) &&
                StringUtils.isNotEmpty(cameraQueryVo.getContent())) {
            sql.append(" and " + cameraQueryVo.getFieldName() + "");
            if (cameraQueryVo.getCondition().equals("equals")) {
                sql.append("= ?");
            }

            if (cameraQueryVo.getCondition().equals("like")) {
                sql.append(" LIKE CONCAT('%',?, '%')");
            }
            total = daosupport.queryForInt(sql.toString(), cameraQueryVo.getContent());
        } else {
            total = daosupport.queryForInt(sql.toString(), null);
        }

        StringBuilder hql = new StringBuilder();
        List<Camera> list;
        hql.append("select * from camera where is_delete=0 and status=0");
        if (StringUtils.isNotEmpty(cameraQueryVo.getFieldName()) &&
                StringUtils.isNotEmpty(cameraQueryVo.getCondition()) &&
                StringUtils.isNotEmpty(cameraQueryVo.getContent())) {
            hql.append(" and " + cameraQueryVo.getFieldName() + "");
            if (cameraQueryVo.getCondition().equals("equals")) {
                hql.append("= ?");
            }

            if (cameraQueryVo.getCondition().equals("like")) {
                hql.append(" LIKE CONCAT('%',?, '%')");
            }
            hql.append(" order by create_time desc limit ?,?");
            list = daosupport.queryForList(hql.toString(), Camera.class, cameraQueryVo.getContent(),
                    cameraQueryVo.getStartNum(), cameraQueryVo.getEndNum());
        } else {
            hql.append(" order by create_time desc limit ?,?");
            list = daosupport.queryForList(hql.toString(), Camera.class, cameraQueryVo.getStartNum(), cameraQueryVo.getEndNum());
        }

        //获取摄像头是否在线状态
        CameraOnlineStatus cameraOnlineStatus = new CameraOnlineStatus();
        for (Camera camera : list) {
            //获取垃圾站名称
            Rubbish rubbish = rubbishMapper.getById(camera.getRubbishId());
            if (null != rubbish) {
                camera.setRubbishName(rubbish.getRubbishName());
            }

            //获取摄像头是否在线状态
            String[] indexCodes = {camera.getCameraNumber()};
            cameraOnlineStatus.setIndexCodes(indexCodes);
            R result = siteWatchOverService.onlineStatus(cameraOnlineStatus);
            if (result != null && result.getCode() == 0) {
                JSONObject jsonObject = JSONObject.parseObject(result.getData().toString());
                List<OnlineVo> onlineVo = JSONObject.parseArray(jsonObject.get("list").toString(), OnlineVo.class);
                if (CollectionUtils.isNotEmpty(onlineVo)) {
                    Integer online = onlineVo.get(0).getOnline();
                    if (null == online) {
                        camera.setStatus(0);
                    } else {
                        camera.setStatus(online);
                    }
                }
            }
        }

        //查询摄像头集合
        PageInfo<Camera> info = new PageInfo<>();
        info.setPageSize(cameraQueryVo.getPageSize());
        info.setPageNum(cameraQueryVo.getPageNo());
        info.setTotal(Long.valueOf(total));
        info.setList(list);
        return R.ok(info);
    }

    @Override
    public R<List<RubbishNameVo>> getRubbishNameList() {
        log.info("ljfl-service[]CameraManagerServiceImpl[]getRubbishNameList[]input.method.getRubbishNameList");
        List<RubbishNameVo> list = rubbishMapper.getRubbishNameList();

        return R.ok(list);
    }

    private Boolean checkCameraNumber(Integer flag, Camera camera) {
        if (StringUtils.isNotEmpty(camera.getCameraNumber())) {
            //判断是新增还是编辑
            if (flag == 1) {
                int count = cameraMapper.countNumber(camera.getCameraNumber());
                if (count > 0) {
                    return false;
                }
            }

            if (flag == 2) {
                int count = cameraMapper.checkNumber(camera.getId(), camera.getCameraNumber());
                if (count > 0) {
                    return false;
                }
            }
        }
        return true;
    }

    //扫描将要报名结束的数据
    private void pageHandle(CameraQueryVo cameraQueryVo) {
        Integer pageNum = cameraQueryVo.getPageNo();
        Integer pageSize = cameraQueryVo.getPageSize();
        if (null == pageSize || pageSize == 0) pageSize = 10;
        if (null == pageNum || pageNum == 0) pageNum = 1;
        Integer endNum = pageSize;
        Integer startNum = (pageNum - CommonAttributes.NUM_ONE) * pageSize;
        cameraQueryVo.setEndNum(endNum);
        cameraQueryVo.setStartNum(startNum);
        cameraQueryVo.setPageNo(pageNum);
        cameraQueryVo.setPageSize(pageSize);
    }
}
