package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.AccountOrganization;
import cn.wisenergy.model.app.AccountUsers;
import cn.wisenergy.model.app.Camera;
import cn.wisenergy.model.app.Rubbish;
import cn.wisenergy.model.vo.*;
import cn.wisenergy.service.app.CommunityBrainService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Slf4j
public class CommunityBrainServiceImpl implements CommunityBrainService {

    @Autowired
    private CameraMapper cameraMapper;

    @Autowired
    private RubbishMapper rubbishMapper;

    @Autowired
    private AccountUsersMapper accountUsersMapper;

    @Autowired
    private AccountOrganizationMapper accountOrganizationMapper;

    @Autowired
    private Info0001Mapper info0001Mapper;

    @Autowired
    private Info0002Mapper info0002Mapper;

    private static final Integer SERVEN_DAY = 7;

    @Override
    public R<CountDetailVo> getCountDetail(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("入参为空!");
        }

        AccountUsers accountUsers = accountUsersMapper.getByAutoId(autoId);
        if (null == accountUsers) {
            return R.ok(new CountDetailVo());
        }

        CountDetailVo countDetailVo = new CountDetailVo();
        //管理员权限
        if (accountUsers.getIsSupper() == 1) {
            //1、垃圾站总数
            int rubbishCount = rubbishMapper.count();
            countDetailVo.setRubbishTotal(rubbishCount);

            //2、总摄像头数
            int cameraCount = cameraMapper.count();
            countDetailVo.setCameraCount(cameraCount);

            //3、街道数量
            int streetCount = accountOrganizationMapper.streetCount();
            countDetailVo.setStreetCount(streetCount);

            //4、社区数量
            int communityCount = accountOrganizationMapper.communityCount();
            countDetailVo.setCommunityCount(communityCount);

            //5、人员总数
            int userCount = accountUsersMapper.count();
            countDetailVo.setUserCount(userCount);
        } else {
            String name = accountUsers.getOrganizationName();
            String organizationAutoId = accountUsers.getOrganizationAutoId();

            //1、垃圾站总数
            int rubbishCount = rubbishMapper.countArea(name);
            countDetailVo.setRubbishTotal(rubbishCount);

            //2、总摄像头数
            //获取垃圾站id
            List<Integer> list = rubbishMapper.getByAreaName(name);
            if (CollectionUtils.isEmpty(list)) {
                countDetailVo.setCameraCount(0);
            } else {
                int cameraCount = 0;
                for (Integer id : list) {
                    cameraCount = cameraCount + cameraMapper.countByRubbishId(id);
                }
                countDetailVo.setCameraCount(cameraCount);
            }


            //3、街道数量
            int streetCount = accountOrganizationMapper.areaLike(name);
            countDetailVo.setStreetCount(streetCount);

            //4、社区数量
            int communityCount;
            if (streetCount != 0) {
                communityCount = accountOrganizationMapper.communityCountByParentId(organizationAutoId);
                countDetailVo.setCommunityCount(communityCount);
            } else {
                communityCount = accountOrganizationMapper.areaLike(name);
                countDetailVo.setCommunityCount(communityCount);
            }

            //5、人员总数
            //判断是小区还是街道
            if (StringUtils.isNotEmpty(accountUsers.getOrganizationAutoId())) {
                AccountOrganization organization = accountOrganizationMapper.getAreaByAutoId(accountUsers.getOrganizationAutoId());
                if (!organization.getRootId().equals("NONE") && organization.getRootId().equals(organization.getParentId())) {
                    //是街道:获取街道下的小区
                    List<String> stringList = accountOrganizationMapper.getByParentId(accountUsers.getOrganizationAutoId());

                    //统计该街道下的小区的人员
                    int count = accountUsersMapper.countByName(stringList);
                    countDetailVo.setUserCount(count);
                }

                if (!organization.getRootId().equals("NONE") && !organization.getRootId().equals(organization.getParentId())) {
                    //是小区，获取小区下的人员
                    List<String> stringList = new ArrayList<>();
                    stringList.add(organization.getName());
                    int count = accountUsersMapper.countByName(stringList);
                    countDetailVo.setUserCount(count);
                }
            } else {
                countDetailVo.setUserCount(1);
            }
        }
        return R.ok(countDetailVo);
    }

    @Override
    public R<List<UserRoleCountVo>> userRoleCount(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("入参为空!");
        }
        AccountUsers accountUsers = accountUsersMapper.getByAutoId(autoId);
        if (null == accountUsers) {
            return R.ok(new ArrayList<>());
        }


        List<UserRoleCountVo> list = new ArrayList<>();
        if (accountUsers.getIsSupper() == 1) {
            list = accountUsersMapper.getUserRoles();
        } else {
            //判断登录用户的组织权限
            //获取组织信息
            AccountOrganization organization = accountOrganizationMapper.getAreaByAutoId(accountUsers.getOrganizationAutoId());
            if (null == organization) {
                //获取当前用户的角色
                UserRoleCountVo roleCountVo = accountUsersMapper.getRoleByAutoId(accountUsers.getAutoId());
                list.add(roleCountVo);
                return R.ok(list);
            }


            //是社区
            if (!organization.getRootId().equals("NONE") && !organization.getRootId().equals(organization.getParentId())) {
                //是社区权限
                //获取社区派单
                List<String> nameList = new ArrayList<>();
                nameList.add(organization.getName());
                list = accountUsersMapper.getByOrgName(nameList);
            }

            if (!organization.getRootId().equals("NONE") && organization.getRootId().equals(organization.getParentId())) {
                //是街道权限: 获取街道下的所有社区
                List<String> streetName = accountOrganizationMapper.getByParentId(organization.getAutoId());
                if (CollectionUtils.isEmpty(streetName)) {
                    //获取当前用户的角色
                    UserRoleCountVo roleCountVo = accountUsersMapper.getRoleByAutoId(accountUsers.getAutoId());
                    list.add(roleCountVo);
                    return R.ok(list);
                }
                list = accountUsersMapper.getByOrgName(streetName);
            }
        }
        return R.ok(list);
    }

    @Override
    public R<List<InspectorRateVo>> getInspectorRate(String autoId, Integer type) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("入参为空!");
        }

        AccountUsers accountUsers = accountUsersMapper.getByAutoId(autoId);
        if (null == accountUsers) {
            return R.ok(new ArrayList<>());
        }

        List<InspectorRateVo> list = new ArrayList<>();
        if (accountUsers.getIsSupper() == 1) {
            //是街道
            if (type == 1) {
                //获取所有街道
                List<String> streetName = accountOrganizationMapper.getStreet();
                if (CollectionUtils.isEmpty(streetName)) {
                    return R.ok(list);
                }
                list.addAll(getRateList(streetName));
            }

            //是社区
            if (type == 2) {
                //获取所有社区
                List<String> communityName = accountOrganizationMapper.getCommunity();
                if (CollectionUtils.isEmpty(communityName)) {
                    return R.ok(list);
                }

                list.addAll(getRateList(communityName));
            }
        } else {
            //街道
            if (type == 1) {
                //获取组织信息
                AccountOrganization organization = accountOrganizationMapper.getAreaByAutoId(accountUsers.getOrganizationAutoId());
                if (null == organization) {
                    return R.error("该用户没有社区权限!");
                }

                //判断账号权限
                if (!organization.getRootId().equals("NONE") && !organization.getRootId().equals(organization.getParentId())) {
                    //是社区权限:获取社区所属街道
                    AccountOrganization organizationStreet = accountOrganizationMapper.getAreaByAutoId(organization.getParentId());
                    if (null == organizationStreet) {
                        return R.error("该用户没有街道权限!");
                    }
                    List<String> name = new ArrayList<>();
                    name.add(organizationStreet.getName());
                    list.addAll(getRateList(name));
                }

                if (!organization.getRootId().equals("NONE") && organization.getRootId().equals(organization.getParentId())) {
                    //是街道权限: 获取街道
                    List<String> name = new ArrayList<>();
                    name.add(organization.getName());
                    list.addAll(getRateList(name));
                }
            }

            //社区
            if (type == 2) {

                //获取组织信息
                AccountOrganization organization = accountOrganizationMapper.getAreaByAutoId(accountUsers.getOrganizationAutoId());
                if (null == organization) {
                    return R.error("该用户没有社区权限!");
                }
                //判断账号权限
                if (!organization.getRootId().equals("NONE") && !organization.getRootId().equals(organization.getParentId())) {
                    //是社区权限
                    //获取社区派单
                    List<String> name = new ArrayList<>();
                    name.add(organization.getName());
                    list.addAll(getRateList(name));
                }

                if (!organization.getRootId().equals("NONE") && organization.getRootId().equals(organization.getParentId())) {
                    //是街道权限: 获取街道下的所有社区
                    List<String> streetName = accountOrganizationMapper.getByParentId(organization.getAutoId());
                    if (CollectionUtils.isEmpty(streetName)) {
                        return R.error("该用户没有社区权限!");
                    }

                    list.addAll(getRateList(streetName));
                }
            }

        }

        list.sort((x, y) -> Double.compare(y.getPercent(), x.getPercent()));
        return R.ok(list);
    }

    @Override
    public R<List<ViolationOrderVo>> getViolationOrder(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("入参为空!");
        }

        AccountUsers accountUsers = accountUsersMapper.getByAutoId(autoId);
        if (null == accountUsers) {
            return R.error("用户信息不存在!");
        }

        List<ViolationOrderVo> list;
        //超级管理员
        if (accountUsers.getIsSupper() == 1) {
            list = info0001Mapper.getAllOrder(null);
        } else {  //普通用户
            list = info0001Mapper.getAllOrder(accountUsers.getNickName());
        }
        //查询督查派单排名
        if (!CollectionUtils.isEmpty(list)) {
            for (ViolationOrderVo violationOrderVo : list) {
                //获取摄像机信息
                QueryWrapper<Camera> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("camera_number", violationOrderVo.getCameraNumber());
                queryWrapper.eq("is_delete", 0);
                Camera camera = cameraMapper.selectOne(queryWrapper);
                if (null != camera) {
                    //获取垃圾站信息
                    Rubbish rubbish = rubbishMapper.getById(camera.getRubbishId());
                    if (null != rubbish) {
                        violationOrderVo.setCommunity(rubbish.getCommunity());
                        violationOrderVo.setRubbishName(rubbish.getRubbishName());
                        violationOrderVo.setStreet(rubbish.getStreet());
                    }
                }
            }
        }

        return R.ok(list);
    }

    @Override
    public R<SendOrderCountVo> getSendOrderCount(String autoId, Integer timeType) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("入参为空!");
        }

        AccountUsers accountUsers = accountUsersMapper.getByAutoId(autoId);
        if (null == accountUsers) {
            return R.error("用户信息为空!");
        }
        //1、统计远程值守派单   0、远程值守派单 1：督查中心派单
        InvestigateVo investigateVo = new InvestigateVo();
        if (accountUsers.getIsSupper() == 1) {
            //统计远程值守派单总数
            int total = info0001Mapper.countSend(0, timeType);

            //统计远程值守派单反馈数量
            int count = info0002Mapper.countBack(0, timeType);

            String percent = "";
            if (count > total || total == 0) {
                percent = "0";
            } else {
                int aa = (int) ((new BigDecimal((float) count / total).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue()) * 100);
                percent = aa + "";
            }

            investigateVo.setTotal(total);
            investigateVo.setBackCount(count);
            investigateVo.setBackPercent(percent);

        } else {
            //根据用户所有的组织权限查询远程值守派单
            if (StringUtils.isBlank(accountUsers.getNickName())) {
                investigateVo.setTotal(0);
                investigateVo.setBackCount(0);
                investigateVo.setBackPercent("0");
            }

            int total = info0001Mapper.sendOrderTotalByType(accountUsers.getNickName(), 0, timeType);
            int count = info0002Mapper.backOrderCountByType(accountUsers.getNickName(), 0, timeType);
            String percent = "";
            if (count > total || total == 0) {
                percent = "0";
            } else {
                int aa = (int) ((new BigDecimal((float) count / total).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue()) * 100);
                percent = aa + "";
            }
            investigateVo.setTotal(total);
            investigateVo.setBackCount(count);
            investigateVo.setBackPercent(percent);
        }


        //2、统计督查中心派单
        InspectorVo inspectorVo = new InspectorVo();
        if (accountUsers.getIsSupper() == 1) {
            //统计远程值守派单总数
            int total = info0001Mapper.countSend(1, timeType);

            //统计远程值守派单反馈数量
            int count = info0002Mapper.countBack(1, timeType);

            String percent = "";
            if (count > total || total == 0) {
                percent = "0";
            } else {
                int aa = (int) ((new BigDecimal((float) count / total).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue()) * 100);
                percent = aa + "";
            }

            inspectorVo.setTotal(total);
            inspectorVo.setBackCount(count);
            inspectorVo.setBackPercent(percent);

        } else {
            //根据用户所有的组织权限查询远程值守派单
            if (StringUtils.isBlank(accountUsers.getNickName())) {
                inspectorVo.setTotal(0);
                inspectorVo.setBackCount(0);
                inspectorVo.setBackPercent("0");
            }

            int total = info0001Mapper.sendOrderTotalByType(accountUsers.getNickName(), 1, timeType);
            int count = info0002Mapper.backOrderCountByType(accountUsers.getNickName(), 1, timeType);
            String percent = "";
            if (count > total || total == 0) {
                percent = "0";
            } else {
                int aa = (int) ((new BigDecimal((float) count / total).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue()) * 100);
                percent = aa + "";
            }
            inspectorVo.setTotal(total);
            inspectorVo.setBackCount(count);
            inspectorVo.setBackPercent(percent);
        }
        SendOrderCountVo sendOrderCountVo = new SendOrderCountVo();
        sendOrderCountVo.setInspectorVo(inspectorVo);
        sendOrderCountVo.setInvestigateVo(investigateVo);
        return R.ok(sendOrderCountVo);
    }

    @Override
    public R<List<ClearTeamVo>> getClearTeam(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("入参为空!");
        }

        AccountUsers accountUsers = accountUsersMapper.getByAutoId(autoId);
        if (null == accountUsers) {
            return R.error("用户信息为空!");
        }

        //获取最近七天的时间
        List<Date> listDate = getDays();
        List<ClearTeamVo> list = new ArrayList<>();
        if (accountUsers.getIsSupper() == 1 || accountUsers.getNickName().equals("清运工")) {
            for (Date dateTime : listDate) {
                //获取接单员为清运员的派单
                int total = info0001Mapper.countByTime(dateTime, "清运工");
                int count = info0002Mapper.countByTime(dateTime, "清运工");
                String percent = "";
                if (count > total || total == 0) {
                    percent = "0";
                } else {
                    int aa = (int) ((new BigDecimal((float) count / total).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue()) * 100);
                    percent = aa + "";
                }
                ClearTeamVo clearTeamVo = new ClearTeamVo();
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
                clearTeamVo.setDayTime(format.format(dateTime));
                clearTeamVo.setClearCount(total);
                clearTeamVo.setClearPercent(percent);
                list.add(clearTeamVo);
            }
        } else {
            //登录账号不是超级管理员，不是清运员
            for (Date dateTime : listDate) {
                ClearTeamVo clearTeamVo = new ClearTeamVo();
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
                clearTeamVo.setDayTime(format.format(dateTime));
                clearTeamVo.setClearCount(0);
                clearTeamVo.setClearPercent("0");
                list.add(clearTeamVo);
            }
        }
        return R.ok(list);
    }


    /**
     * 获取过去7天内的日期数组
     *
     * @return 日期数组
     */
    private ArrayList<Date> getDays() {
        ArrayList<Date> pastDaysList = new ArrayList<>();
        for (int i = CommunityBrainServiceImpl.SERVEN_DAY - 1; i >= 0; i--) {
            pastDaysList.add(getPastDate(i));
        }
        return pastDaysList;
    }

    /**
     * 获取过去第几天的日期
     *
     * @param past 天
     * @return 日期
     */
    private Date getPastDate(int past) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_YEAR, calendar.get(Calendar.DAY_OF_YEAR) - past);
        Date today = calendar.getTime();
        log.info("");
        return today;
    }

    private List<InspectorRateVo> getRateList(List<String> areaList) {
        List<InspectorRateVo> result = new ArrayList<>();
        for (String name : areaList) {
            InspectorRateVo rateVo = new InspectorRateVo();
            rateVo.setAreaName(name);

            //统计社区督查派单总数
            int total = info0001Mapper.sendOrderTotal(name);

            //统计社区督查派单反馈数量
            int count = info0002Mapper.backOrderCount(name);
            String percent = "";
            if (count > total || total == 0) {
                percent = "0";
            } else {
                double aa = (double) ((new BigDecimal((float) count / total).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue()) * 100);
                percent = aa + "";
            }
            rateVo.setPercent(Double.valueOf(percent));
            result.add(rateVo);
        }

        return result;
    }

}
