package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.constant.CommonAttributes;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.NewContentMapper;
import cn.wisenergy.model.app.NewContent;
import cn.wisenergy.model.vo.*;
import cn.wisenergy.service.app.Daosupport;
import cn.wisenergy.service.app.NewContentService;
import cn.wisenergy.service.common.Common;
import cn.wisenergy.service.util.HttpsUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.client.RestTemplate;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class NewContentServiceImpl extends ServiceImpl<NewContentMapper, NewContent> implements NewContentService {
    @Autowired
    private NewContentMapper newContentMapper;

    @Autowired
    private Daosupport daosupport;

    @Autowired
    private RestTemplate restTemplate;

    @Transactional
    @Override
    public R<NewContent> add(NewContent newContent, String token) {
        if (null == newContent) {
            return R.error("入参为空!");
        }
        newContent.setIsDelete(0);

        //推送到公众号
        NewMessageVo messageVo = new NewMessageVo();
        List<WxRequestVo> list = new ArrayList<>();
        WxRequestVo wxRequestVo = new WxRequestVo();
        wxRequestVo.setAuthod("");
        wxRequestVo.setContent(newContent.getContent());
        wxRequestVo.setContent_source_url("http://www.baidu.com");

        //封面素材id
        wxRequestVo.setThumb_media_id(newContent.getMediaId());
        wxRequestVo.setNeed_open_comment(0);
        wxRequestVo.setOnly_fans_can_comment(0);
        wxRequestVo.setShow_cover_pic("1");
        wxRequestVo.setTitle(newContent.getTitle());
        list.add(wxRequestVo);
        messageVo.setArticles(list);
        UploadMaterialVo mediaId = addNew(messageVo, token);
        if (StringUtils.isNotEmpty(mediaId.getErrcode())) {
            return R.error("上传图文消息失败!");
        }
        log.info("图片封面media_id:" + newContent.getMediaId());
        log.info("永久素材media_id:" + mediaId.getMedia_id());
        //存储永久素材id
        newContent.setMediaId(mediaId.getMedia_id());
        int count = newContentMapper.add(newContent);
        if (count == 0) {
            return R.error("添加失败！");
        }

        return R.ok(newContent);
    }

    @Override
    public R<Boolean> edit(NewContent newContent, String token) {
        if (null == newContent) {
            return R.error("入参为空!");
        }

        //获取要修改的对象
        NewContent result = newContentMapper.getById(newContent.getId(), newContent.getType());
        if (null == result) {
            return R.error("信息不存在!");
        }

        result.setContent(newContent.getContent());
        result.setTitle(newContent.getTitle());

        //构造参数
        RequestMaterialVo request = new RequestMaterialVo();
        if (StringUtils.isBlank(result.getMediaId())) {
            return R.error("素材信息不存在!");
        }
        request.setMedia_id(result.getMediaId());

        //获取永久素材
        QueryMaterialVo queryMaterialVo = getMaterial(request, token);
        if (StringUtils.isNotEmpty(queryMaterialVo.getErrcode())) {
            return R.error("获取素材信息失败!");
        }

        boolean bool = updateNewContent(newContent, token, queryMaterialVo, result.getMediaId());
        if (!bool) {
            return R.ok(1, false);
        }

        return R.ok(0, true);
    }

    @Override
    public R<Boolean> delete(Integer id, Integer type) {
        if (null == id || null == type) {
            return R.error("入参为空!");
        }

        //获取要修改的对象
        NewContent result = newContentMapper.getById(id, type);
        if (null == result) {
            return R.error("信息不存在!");
        }

        result.setIsDelete(1);
        int count = newContentMapper.edit(result);
        if (count == 0) {
            return R.ok(1, false);
        }
        return R.ok(0, true);
    }

    @Override
    public R<PageInfo> getList(NewContentQueryVo queryVo) {
        log.info("ljfl-service[]NewContentServiceImpl[]getList[]input.param.queryVo:{}" + queryVo);
        if (null == queryVo) {
            return R.error("入参为空");
        }
        pageHandle(queryVo);
        StringBuilder sql = new StringBuilder();
        Integer total;
        sql.append("select count(1) from new_content where is_delete=0 and type = " + queryVo.getType() + " ");
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(queryVo.getFieldName()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(queryVo.getCondition()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(queryVo.getContent())) {
            sql.append(" and " + queryVo.getFieldName() + "");
            if (queryVo.getCondition().equals("equals")) {
                sql.append("= ?");
            }

            if (queryVo.getCondition().equals("like")) {
                sql.append(" LIKE CONCAT('%', ?, '%')");
            }

            total = daosupport.queryForInt(sql.toString(), queryVo.getContent());
        } else {
            total = daosupport.queryForInt(sql.toString(), null);
        }

        StringBuilder hql = new StringBuilder();
        List<NewContent> list;
        hql.append("select * from new_content where is_delete=0 and type = " + queryVo.getType() + "");
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(queryVo.getFieldName()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(queryVo.getCondition()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(queryVo.getContent())) {
            hql.append(" and " + queryVo.getFieldName() + "");
            if (queryVo.getCondition().equals("equals")) {
                hql.append("= ?");
            }

            if (queryVo.getCondition().equals("like")) {
                hql.append(" LIKE CONCAT('%',?, '%')");
            }
            hql.append(" order by create_time desc limit ?,?");
            list = daosupport.queryForList(hql.toString(), NewContent.class, queryVo.getContent(),
                    queryVo.getStartNum(), queryVo.getEndNum());
        } else {
            hql.append(" order by create_time desc limit ?,?");
            list = daosupport.queryForList(hql.toString(), NewContent.class, queryVo.getStartNum(),
                    queryVo.getEndNum());
        }

        PageInfo<NewContent> info = new PageInfo<>();
        info.setPageSize(queryVo.getPageSize());
        info.setPageNum(queryVo.getPageNo());
        info.setTotal(total);
        info.setList(list);
        return R.ok(info);
    }

    @Override
    public AccessToken getToken() {
        //封装入参
        String grant_type = "client_credential";
        String app_id = Common.APP_ID;
        String app_seCRET = Common.APP_SECRET;
        String url = Common.ACCESS_TOKEN_URL + "?grant_type=" + grant_type + "&appid=" + app_id + "&secret=" + app_seCRET;

        String token = this.restTemplate.getForObject(url, String.class);
        AccessToken accessToken = JSONObject.parseObject(token, AccessToken.class);
        log.info("");
        return accessToken;
    }

    @Override
    public UploadMaterialVo addNew(NewMessageVo messageVo, String accessToken) {
        log.info("NewContentServiceImpl[]addNew[]input.param:{}", messageVo);
        if (null == messageVo) {
            return new UploadMaterialVo();
        }

        final HttpHeaders requestHeaders = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        requestHeaders.setContentType(type);

        //封装入参
        JSONObject args = new JSONObject(true);
        args.put("articles", messageVo.getArticles());

        String url = Common.ADD_NEWS_URL + "?access_token=" + accessToken;
        HttpEntity<String> formEntity = new HttpEntity<>(args.toJSONString(), requestHeaders);
        ResponseEntity<String> responseEntity = this.restTemplate.postForEntity(url, formEntity, String.class);
        if (StringUtils.isBlank(responseEntity.getBody())) {
            return new UploadMaterialVo();
        }
        UploadMaterialVo uploadMaterialVo = JSONObject.parseObject(responseEntity.getBody(), UploadMaterialVo.class);
        log.info("NewContentServiceImpl[]addNew[]uploadMaterialVo:{}" + uploadMaterialVo);
        return uploadMaterialVo;
    }

    @Override
    public UpdateMaterialVo updateMaterial(NewMessageVo messageVo, String accessToken) {
        log.info("NewContentServiceImpl[]updateMaterial[]input.param:{}", messageVo);
        if (null == messageVo) {
            return new UpdateMaterialVo();
        }

        final HttpHeaders requestHeaders = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        requestHeaders.setContentType(type);

        //封装入参
        JSONObject args = new JSONObject(true);
        args.put("media_id", messageVo.getMedia_id());
        args.put("index", messageVo.getIndex());
        args.put("articles", messageVo.getArticles());
        String url = Common.UPDATE_NEWS_URL + "?access_token=" + accessToken;
        HttpEntity<String> formEntity = new HttpEntity<>(args.toJSONString(), requestHeaders);
        ResponseEntity<String> responseEntity = this.restTemplate.postForEntity(url, formEntity, String.class);
        if (StringUtils.isBlank(responseEntity.getBody())) {
            return new UpdateMaterialVo();
        }

        UpdateMaterialVo updateMaterialVo = JSONObject.parseObject(responseEntity.getBody(), UpdateMaterialVo.class);
        log.info("");
        return updateMaterialVo;
    }

    @Override
    public Object list(WxDto wxDto, String accessToken) {

        final HttpHeaders requestHeaders = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        requestHeaders.setContentType(type);

        //封装入参
        JSONObject args = new JSONObject(true);
        args.put("type", wxDto.getType());
        args.put("offset", wxDto.getOffset());
        args.put("count", wxDto.getCount());
        log.info("打印入参数据：" + args.toJSONString());
        String url = "https://api.weixin.qq.com/cgi-bin/material/batchget_material" + "?access_token=" + accessToken;
        HttpEntity<String> formEntity = new HttpEntity<>(args.toJSONString(), requestHeaders);
        ResponseEntity<String> responseEntity = this.restTemplate.postForEntity(url, formEntity, String.class);
        return R.ok(responseEntity.getBody());
    }

    @Override
    public R<UploadMaterialVo> addMaterialEver(String fileUrl, String type, String accessToken) {
        if (StringUtils.isBlank(fileUrl) || StringUtils.isBlank(type)) {
            return R.error("入参为空!");
        }

        try {
            File file = new File(fileUrl);

            //接口请求路径
            String url = Common.ADD_MATERIAL + "?access_token=" + accessToken + "&type=" + type;

            //调用form-date请求
            String result = HttpsUtil.post(url, null, file);
            result = result.replaceAll("[\\\\]", "");
            System.out.println("result:" + result);
            UploadMaterialVo resultJSON = JSONObject.parseObject(result, UploadMaterialVo.class);
            if (resultJSON != null) {
                if (StringUtils.isNotEmpty(resultJSON.getMedia_id())) {
                    return R.ok(resultJSON);
                } else {
                    return R.error("上传" + type + "失败!");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return R.error("上传" + type + "失败!");
    }

    @Override
    public QueryMaterialVo getMaterial(RequestMaterialVo query, String accessToken) {
        if (null == query || StringUtils.isBlank(query.getMedia_id())) {
            return new QueryMaterialVo();
        }

        final HttpHeaders requestHeaders = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        requestHeaders.setContentType(type);

        //封装入参
        JSONObject args = new JSONObject(true);
        args.put("media_id", query.getMedia_id());
        String url = Common.GET_MATERIAL + "?access_token=" + accessToken;
        HttpEntity<String> formEntity = new HttpEntity<>(args.toJSONString(), requestHeaders);
        ResponseEntity<String> responseEntity = this.restTemplate.postForEntity(url, formEntity, String.class);
        if (StringUtils.isBlank(responseEntity.getBody())) {
            return new QueryMaterialVo();
        }

        QueryMaterialVo newMessageVo = JSONObject.parseObject(responseEntity.getBody(), QueryMaterialVo.class);
        log.info("");
        return newMessageVo;
    }

    //扫描将要报名结束的数据
    private void pageHandle(NewContentQueryVo queryVo) {
        Integer pageNum = queryVo.getPageNo();
        Integer pageSize = queryVo.getPageSize();
        if (null == pageSize || pageSize == 0) pageSize = 10;
        if (null == pageNum || pageNum == 0) pageNum = 1;
        Integer endNum = pageSize;
        Integer startNum = (pageNum - CommonAttributes.NUM_ONE) * pageSize;
        queryVo.setEndNum(endNum);
        queryVo.setStartNum(startNum);
        queryVo.setPageNo(pageNum);
        queryVo.setPageSize(pageSize);
    }

    @Transactional
    public boolean updateNewContent(NewContent newContent, String token, QueryMaterialVo query, String updateMediaId) {
        if (null == newContent || null == query ||
                CollectionUtils.isEmpty(query.getNews_item()) ||
                StringUtils.isBlank(updateMediaId)
        ) {
            return false;
        }

        //编辑新闻/使用手册
        int count = newContentMapper.edit(newContent);
        if (count == 0) {
            return false;
        }

        //编辑图文素材
        //构造参数
        NewMessageVo messageVo = new NewMessageVo();
        WxRequestVo wxRequestVo = query.getNews_item().get(0);
        messageVo.setMedia_id(updateMediaId);
        messageVo.setIndex(0);

        List<WxRequestVo> list = new ArrayList<>();
        WxRequestVo newInfo = new WxRequestVo();
        newInfo.setTitle(wxRequestVo.getTitle());
        newInfo.setShow_cover_pic("1");
        newInfo.setOnly_fans_can_comment(0);
        newInfo.setNeed_open_comment(0);
        newInfo.setThumb_media_id(newContent.getMediaId());
        newInfo.setContent_source_url(wxRequestVo.getContent_source_url());
        newInfo.setAuthod(wxRequestVo.getAuthod());
        newInfo.setDigest(wxRequestVo.getDigest());
        list.add(newInfo);
        messageVo.setArticles(list);
        UpdateMaterialVo materialVo = updateMaterial(messageVo, token);
        if (StringUtils.isNotEmpty(materialVo.getErrcode())) {
            return false;
        }

        return true;
    }
}
