package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.constant.CommonAttributes;
import cn.wisenergy.common.expection.BaseException;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.ResidentMapper;
import cn.wisenergy.model.app.Resident;
import cn.wisenergy.model.vo.ResidentQueryVo;
import cn.wisenergy.service.app.Daosupport;
import cn.wisenergy.service.app.ResidentService;
import cn.wisenergy.service.excel.listener.ResidentListener;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class ResidentServiceImpl extends ServiceImpl<ResidentMapper, Resident> implements ResidentService {

    @Autowired
    private ResidentMapper residentMapper;

    @Autowired
    private Daosupport daosupport;

    @Override
    public R<Resident> add(Resident resident) {
        log.info("ljfl-service[]ResidentServiceImpl[]add[]input.param.resident:{}" + resident);
        if (null == resident) {
            return R.error("入参为空");
        }

        //1、检查手机号是否重复
        //flag为1，代表新增，flag为2代表编辑
        Integer flag = 1;
        this.checkCameraPhone(flag, resident);

        //2、构造初始化参数
        resident.setIsDelete(0);
        resident.setCreateTime(new Date());
        resident.setUpdateTime(new Date());

        int bool = residentMapper.add(resident);
        if (bool == 0) {
            return R.error("添加摄像头失败");
        }
        return R.ok(resident);
    }

    @Override
    public R<Boolean> edit(Resident resident) {
        log.info("ljfl-service[]ResidentServiceImpl[]edit[]input.param.resident:{}" + resident);
        if (null == resident) {
            return R.error("入参为空");
        }

        //1、检查手机号是否重复
        //flag为1，代表新增，flag为2代表编辑
        Integer flag = 2;
        this.checkCameraPhone(flag, resident);

        //编辑居民信息
        Resident residentReal = residentMapper.selectById(resident.getId());
        residentReal.setName(resident.getName());
        residentReal.setPhone(resident.getPhone());
        residentReal.setRemark(resident.getRemark());
        residentReal.setSex(resident.getSex());
        residentReal.setCommunity(resident.getCommunity());
        residentReal.setStreet(resident.getStreet());
        int count = residentMapper.modify(resident);
        if (count == 0) {
            return R.ok(1, false);
        }
        return R.ok(0, true);
    }

    @Override
    public R<Boolean> delete(Integer id) {
        log.info("ljfl-service[]ResidentServiceImpl[]delete[]input.param.id:{}" + id);
        if (null == id) {
            return R.error("入参为空");
        }

        //根据id，获取居民信息
        Resident resident = residentMapper.selectById(id);
        if (null != resident) {
            resident.setIsDelete(1);
            int count = residentMapper.updateById(resident);
            if (count != 0) {
                return R.ok(0, true);
            }
        }
        return R.ok(1, false);
    }

    @Override
    public R<PageInfo<Resident>> getList(ResidentQueryVo residentQueryVo) {
        log.info("ljfl-service[]ResidentServiceImpl[]getList[]input.param.residentQueryVo:{}" + residentQueryVo);
        if (null == residentQueryVo) {
            return R.error("入参为空");
        }
        pageHandle(residentQueryVo);
        StringBuilder sql = new StringBuilder();
        Integer total;
        sql.append("select count(1) from resident where is_delete=0 ");
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(residentQueryVo.getFieldName()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(residentQueryVo.getCondition()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(residentQueryVo.getContent())) {
            sql.append(" and " + residentQueryVo.getFieldName() + "");
            if (residentQueryVo.getCondition().equals("equals")) {
                sql.append("= ?");
            }

            if (residentQueryVo.getCondition().equals("like")) {
                sql.append(" LIKE CONCAT('%', ?, '%')");
            }

            if (residentQueryVo.getFieldName().equals("sex")) {
                if (residentQueryVo.getContent().equals("男")) {
                    residentQueryVo.setContent("1");
                }
                if (residentQueryVo.getContent().equals("女")) {
                    residentQueryVo.setContent("2");
                }
            }
            total = daosupport.queryForInt(sql.toString(), residentQueryVo.getContent());
        } else {
            total = daosupport.queryForInt(sql.toString(), null);
        }

        StringBuilder hql = new StringBuilder();
        List<Resident> list;
        hql.append("select * from resident where is_delete=0");
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(residentQueryVo.getFieldName()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(residentQueryVo.getCondition()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(residentQueryVo.getContent())) {
            hql.append(" and " + residentQueryVo.getFieldName() + "");
            if (residentQueryVo.getCondition().equals("equals")) {
                hql.append("= ?");
            }

            if (residentQueryVo.getCondition().equals("like")) {
                hql.append(" LIKE CONCAT('%',?, '%')");
            }
            hql.append(" order by create_time desc limit ?,?");
            list = daosupport.queryForList(hql.toString(), Resident.class, residentQueryVo.getContent(),
                    residentQueryVo.getStartNum(), residentQueryVo.getEndNum());
        } else {
            hql.append(" order by create_time desc limit ?,?");
            list = daosupport.queryForList(hql.toString(), Resident.class, residentQueryVo.getStartNum(),
                    residentQueryVo.getEndNum());
        }

        PageInfo<Resident> info = new PageInfo<>();
        info.setPageSize(residentQueryVo.getPageSize());
        info.setPageNum(residentQueryVo.getPageNo());
        info.setTotal(total);
        info.setList(list);
        return R.ok(info);
    }


    private void checkCameraPhone(Integer flag, Resident resident) {
        if (flag == 1) {
            if (StringUtils.isNotEmpty(resident.getPhone())) {
                int count = residentMapper.countPhone(resident.getPhone());
                //判断是新增还是编辑
                if (count > 0) {
                    throw new RuntimeException("居民" + resident.getName() + "的手机号重复！");
                }
            }
        }

        if (flag == 2) {
            int count = residentMapper.checkPhone(resident.getId(), resident.getPhone());
            if (count > 0) {
                throw new RuntimeException("居民" + resident.getName() + "手机号重复！");
            }
        }
    }

    //扫描将要报名结束的数据
    private void pageHandle(ResidentQueryVo residentQueryVo) {
        Integer pageNum = residentQueryVo.getPageNo();
        Integer pageSize = residentQueryVo.getPageSize();
        if (null == pageSize || pageSize == 0) pageSize = 10;
        if (null == pageNum || pageNum == 0) pageNum = 1;
        Integer endNum = pageSize;
        Integer startNum = (pageNum - CommonAttributes.NUM_ONE) * pageSize;
        residentQueryVo.setEndNum(endNum);
        residentQueryVo.setStartNum(startNum);
        residentQueryVo.setPageNo(pageNum);
        residentQueryVo.setPageSize(pageSize);
    }

    /**
     * Excel批量添加社区居民
     *
     * @param file
     * @return
     */
    @Override
    public void excelAdd(MultipartFile file, HttpServletResponse response) throws IOException {
        //判断文件类型=
        boolean valid = this.validContentType(file.getOriginalFilename());
        if (!valid) {
            throw new BaseException("请传入Excel文件");
        }

        //读取数据进行入库操作
        EasyExcel.read(file.getInputStream(), Resident.class, new ResidentListener(this)).sheet().doRead();

    }

    public boolean validContentType(String fileName) {
        if (StringUtils.isBlank(fileName)) {
            return false;
        }
        String substring = fileName.substring(fileName.lastIndexOf(".") + 1);
        if (substring.equals("xlsx") || substring.equals("xls")) {
            return true;
        }
        return false;
    }

    /**
     * 批量添加社区居民Excel模板
     *
     * @param response
     */
    @Override
    public void excelTemplate(HttpServletResponse response) throws IOException {
        //构造假数据
        List<Resident> residents = this.mockData();
        //excel导出
        this.download(response, Resident.class, residents, "居民管理模板", "居民管理");

    }

    @Override
    public Resident getByPhone(String phone) {
        Resident resident = residentMapper.getByPhone(phone);
        log.info("");
        return resident;
    }

    private List<Resident> mockData() {
        ArrayList<Resident> residents = new ArrayList<>(2);
        //模拟两条假数据
        for (int i = 0; i < 2; i++) {
            Resident build = Resident.builder()
                    .name("王武" + i)
                    .sex(i + 1)
                    .phone("1352658454" + i + 1)
                    .street("北下关街道")
                    .community("气象局小区")
                    .remark("备注" + i)
                    .build();
            residents.add(build);
        }
        return residents;
    }

    /**
     * 导出数据
     *
     * @param response
     * @param clazz     实体类对象的类型
     * @param data      导出的数据
     * @param fileName  工作簿文件名
     * @param sheetName 工作表文件名
     * @throws IOException
     */
    private void download(HttpServletResponse response, Class clazz, List data, String fileName, String sheetName) throws IOException {
        if (StringUtils.isBlank(fileName)) {
            fileName = "测试";
        }
        if (StringUtils.isBlank(sheetName)) {
            sheetName = "模板";
        }
        // 这里注意 有同学反应使用swagger 会导致各种问题，请直接用浏览器或者用postman
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("NewContent-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");

        EasyExcel.write(response.getOutputStream(), clazz).sheet(sheetName).doWrite(data);
    }
}
