package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.constant.CommonAttributes;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.AccountUsersMapper;
import cn.wisenergy.mapper.CameraMapper;
import cn.wisenergy.mapper.RubbishMapper;
import cn.wisenergy.model.app.AccountUsers;
import cn.wisenergy.model.app.Camera;
import cn.wisenergy.model.app.Rubbish;
import cn.wisenergy.model.vo.RubbishQueryVo;
import cn.wisenergy.service.app.Daosupport;
import cn.wisenergy.service.app.RubbishService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class RubbishServiceImpl extends ServiceImpl<RubbishMapper, Rubbish> implements RubbishService {
    @Autowired
    private RubbishMapper rubbishMapper;

    @Autowired
    private Daosupport daosupport;

    @Autowired
    private CameraMapper cameraMapper;

    @Autowired
    private AccountUsersMapper usersMapper;

    @Override
    public R<Boolean> add(Rubbish rubbish) {
        log.info("ljfl-service[]RubbishServiceImpl[]add[]input.param.rubbish:{}" + rubbish);
        if (null == rubbish) {
            return R.error("入参为空");
        }

        //1、检查垃圾站编号是否重复
        //flag为1，代表新增，flag为2代表编辑
        Integer flag = 1;
        Boolean bool = checkRubbishNumber(flag, rubbish);
        if (!bool) {
            return R.error("垃圾站编号重复！");
        }

        //获取负责人姓名
        AccountUsers accountUsers = usersMapper.getById(rubbish.getUserId());
        rubbish.setUserName(accountUsers.getNickName());

        //初始化垃圾站信息
        rubbish.setStatus(0);
        rubbish.setIsDelete(0);
        int count = rubbishMapper.add(rubbish);
        if (count == 0) {
            return R.ok(1, false);
        }
        return R.ok(0, true);
    }

    @Override
    public R<Boolean> edit(Rubbish rubbish) {
        log.info("ljfl-service[]RubbishServiceImpl[]edit[]input.param.rubbish:{}" + rubbish);
        if (null == rubbish) {
            return R.error("入参为空");
        }

        //1、检查垃圾站编号是否重复
        //flag为1，代表新增，flag为2代表编辑
        Integer flag = 2;
        Boolean bool = checkRubbishNumber(flag, rubbish);
        if (!bool) {
            return R.error("垃圾站编号重复！");
        }


        //2、根据id，获取垃圾站信息
        Rubbish rubbishReal = rubbishMapper.selectById(rubbish.getId());

        //获取负责人姓名
        AccountUsers accountUsers = usersMapper.getById(rubbish.getUserId());
        rubbishReal.setUserName(accountUsers.getNickName());

        //3、编辑修改内容
        rubbishReal.setClickTime(rubbish.getClickTime());
        rubbishReal.setCommunity(rubbish.getCommunity());
        rubbishReal.setLiveTime(rubbish.getLiveTime());
        rubbishReal.setRemark(rubbish.getRemark());
        rubbishReal.setRubbishNumber(rubbish.getRubbishNumber());
        rubbishReal.setStreet(rubbish.getStreet());
        rubbishReal.setUserId(rubbish.getUserId());
        rubbishReal.setRubbishName(rubbish.getRubbishName());
        int count = rubbishMapper.modify(rubbishReal);
        if (count == 0) {
            return R.ok(1, false);
        }
        return R.ok(0, true);
    }

    @Override
    public R<Boolean> delete(Integer id) {
        log.info("ljfl-service[]RubbishServiceImpl[]delete[]input.param.id:{}" + id);
        if (null == id) {
            return R.error("入参为空");
        }

        //2、根据id，获取垃圾站信息
        Rubbish rubbish = rubbishMapper.selectById(id);
        if (null != rubbish) {

            //获取摄像头信息
            List<Camera> cameras = cameraMapper.getByRubbishId(rubbish.getId());
            if (CollectionUtils.isNotEmpty(cameras)) {
                for (Camera camera : cameras) {
                    if (camera.getIsDelete() == 0) {
                        return R.error("该垃圾站已关联摄像头，无法删除！");
                    }
                }
            }
            rubbish.setIsDelete(1);
            int count = rubbishMapper.updateById(rubbish);
            if (count == 1) {
                return R.ok(0, true);
            }
        }
        return R.ok(1, false);
    }

    @Override
    public R<PageInfo<Rubbish>> getList(RubbishQueryVo rubbishQueryVo) {
        log.info("ljfl-service[]RubbishServiceImpl[]getList[]input.param.rubbishQueryVo:{}" + rubbishQueryVo);
        if (null == rubbishQueryVo) {
            return R.error("入参为空");
        }
        pageHandle(rubbishQueryVo);

        StringBuilder sql = new StringBuilder();
        Integer total;
        sql.append("select count(1) from rubbish where is_delete=0 and status=0");
        if (StringUtils.isNotEmpty(rubbishQueryVo.getFieldName()) &&
                StringUtils.isNotEmpty(rubbishQueryVo.getCondition()) &&
                StringUtils.isNotEmpty(rubbishQueryVo.getContent())) {
            sql.append(" and " + rubbishQueryVo.getFieldName() + "");
            if (rubbishQueryVo.getCondition().equals("equals")) {
                sql.append("= ?");
            }

            if (rubbishQueryVo.getCondition().equals("like")) {
                sql.append(" LIKE CONCAT('%', ?, '%')");
            }
            total = daosupport.queryForInt(sql.toString(), rubbishQueryVo.getContent());
        } else {
            //统计数据总条数
            total = daosupport.queryForInt(sql.toString(), null);
        }

        StringBuilder hql = new StringBuilder();
        List<Rubbish> list;
        hql.append("select * from rubbish where is_delete=0 and status=0");
        if (StringUtils.isNotEmpty(rubbishQueryVo.getFieldName()) &&
                StringUtils.isNotEmpty(rubbishQueryVo.getCondition()) &&
                StringUtils.isNotEmpty(rubbishQueryVo.getContent())) {
            hql.append(" and " + rubbishQueryVo.getFieldName() + "");
            if (rubbishQueryVo.getCondition().equals("equals")) {
                hql.append("= ?");
            }

            if (rubbishQueryVo.getCondition().equals("like")) {
                hql.append(" LIKE CONCAT('%', ?, '%')");
            }
            hql.append(" order by create_time desc limit ?,?");
            list = daosupport.queryForList(hql.toString(), Rubbish.class, rubbishQueryVo.getContent(),
                    rubbishQueryVo.getStartNum(), rubbishQueryVo.getEndNum());
        } else {
            hql.append(" order by create_time desc limit ?,?");
            list = daosupport.queryForList(hql.toString(), Rubbish.class,
                    rubbishQueryVo.getStartNum(), rubbishQueryVo.getEndNum());
        }

        for (Rubbish rubbish : list) {
            //获取垃圾站下摄像头数量
            int count = cameraMapper.countByRubbishId(rubbish.getId());
            rubbish.setCount(count);
        }
        PageInfo<Rubbish> info = new PageInfo<>();
        info.setPageSize(rubbishQueryVo.getPageSize());
        info.setPageNum(rubbishQueryVo.getPageNo());
        info.setTotal(Long.valueOf(total));
        info.setList(list);
        return R.ok(info);
    }

    private Boolean checkRubbishNumber(Integer flag, Rubbish rubbish) {
        if (StringUtils.isNotEmpty(rubbish.getRubbishNumber())) {
            //判断是新增还是编辑
            if (flag == 1) {
                int count = rubbishMapper.countNumber(rubbish.getRubbishNumber());
                if (count > 0) {
                    return false;
                }
            }

            if (flag == 2) {
                int count = rubbishMapper.checkNumber(rubbish.getId(), rubbish.getRubbishNumber());
                if (count > 0) {
                    return false;
                }
            }
        }
        return true;
    }

    //扫描将要报名结束的数据
    private void pageHandle(RubbishQueryVo rubbishQueryVo) {
        Integer pageNum = rubbishQueryVo.getPageNo();
        Integer pageSize = rubbishQueryVo.getPageSize();
        if (null == pageSize || pageSize == 0) pageSize = 10;
        if (null == pageNum || pageNum == 0) pageNum = 1;
        Integer endNum = pageSize;
        Integer startNum = (pageNum - CommonAttributes.NUM_ONE) * pageSize;
        rubbishQueryVo.setEndNum(endNum);
        rubbishQueryVo.setStartNum(startNum);
        rubbishQueryVo.setPageNo(pageNum);
        rubbishQueryVo.setPageSize(pageSize);
    }
}
