package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.constant.CommonAttributes;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.*;
import cn.wisenergy.model.vo.*;
import cn.wisenergy.service.app.Daosupport;
import cn.wisenergy.service.app.SendOrderService;
import cn.wisenergy.service.common.Common;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.text.SimpleDateFormat;
import java.util.*;


/**
 * @ Description   :  派单业务逻辑实现
 * @ Author        :  lcc
 * @ CreateDate    :  2020/11/5 16:13
 */
@Service
@Slf4j
public class SendOrderServiceImpl implements SendOrderService {

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private Info0001Mapper info0001Mapper;

    @Autowired
    private Info0002Mapper info0002Mapper;

    @Autowired
    private AccountUsersMapper accountUsersMapper;

    @Autowired
    private Daosupport daosupport;

    @Autowired
    private SystemRolesMapper rolesMapper;

    @Autowired
    private CameraMapper cameraMapper;

    @Autowired
    private RubbishMapper rubbishMapper;

    @Autowired
    private Info0001SystemNodeTaskMapper taskMapper;

    @Override
    public R<JSONObject> sendOrder(SendQuestVo sendQuestVo) {
        log.info("ljfl-service[]SendOrderServiceImpl[]sendOrder[]input.param.orderDetailVo：{}" + sendQuestVo);
        if (null == sendQuestVo ||
                StringUtils.isBlank(sendQuestVo.getServiceNeed()) ||
                StringUtils.isBlank(sendQuestVo.getNotice())
        ) {
            return R.error("参数错误");
        }

        //1、获取当前登录用户信息
        AccountUsers accountUsers = accountUsersMapper.getByAutoId(sendQuestVo.getAutoId());
        if (null == accountUsers || StringUtils.isBlank(accountUsers.getAutoId())) {
            return R.error("用户信息不能为空！");
        }
        String nickName = accountUsers.getNickName();
        final HttpHeaders requestHeaders = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        requestHeaders.setContentType(type);
        JSONObject args = new JSONObject(true);
        args.put("create_user_auto_id", nickName);
        args.put("update_user_auto_id", nickName);
        args.put("field_101", sendQuestVo.getServiceNeed());
        args.put("field_102", sendQuestVo.getImageOne());
        args.put("field_103", sendQuestVo.getImageTwo());
        args.put("field_104", sendQuestVo.getImageThree());
        args.put("field_105", nickName);
        args.put("field_106", sendQuestVo.getReceiveName());
        args.put("field_107", sendQuestVo.getNotice());
        args.put("field_108", new Date());

        //根据摄像机编号，获取摄像机信息
        QueryWrapper<Camera> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("camera_number", sendQuestVo.getCameraNumber());
        queryWrapper.eq("is_delete", 0);
        Camera camera = cameraMapper.selectOne(queryWrapper);
        if (null == camera) {
            return R.error("该摄像头未添加,请到系统设置-摄像头管理添加该摄像头");
        }

        //获取垃圾站位置
        Rubbish rubbish = rubbishMapper.getById(camera.getRubbishId());
        String site = "";
        if (null != rubbish) {
            if (null != rubbish.getStreet()) {
                site = site + rubbish.getStreet();
            }

            if (null != rubbish.getCommunity()) {
                site = site + "-" + rubbish.getCommunity();
            }

            if (null != rubbish.getRubbishName()) {
                site = site + "-" + rubbish.getRubbishName();
            }
        } else {
            return R.error("该摄像头所属垃圾站不存在");

        }
        args.put("field_109", site);
        args.put("field_110", sendQuestVo.getReceivePhone());
        args.put("field_111", sendQuestVo.getUploadVideo());
        args.put("field_112", sendQuestVo.getSendType().toString());
        args.put("field_113", sendQuestVo.getCameraNumber());
        args.put("field_114", null);
        args.put("field_115", null);
        args.put("user_list", sendQuestVo.getAccountUsers());
        HttpEntity<String> formEntity = new HttpEntity<>(args.toJSONString(), requestHeaders);
        ResponseEntity<JSONObject> responseEntity = this.restTemplate.postForEntity(Common.SEND_ORDER_URL, formEntity, JSONObject.class);
        return R.ok(responseEntity.getBody());
    }

    @Override
    public R<JSONObject> receiveOrder(ReceiveOrderVo receiveOrderVo) {
        log.info("ljfl-service[]SendOrderServiceImpl[]receiveOrder[]input.param.receiveOrderVo：{}" + receiveOrderVo);
        if (null == receiveOrderVo || StringUtils.isBlank(receiveOrderVo.getServiceAutoId())) {
            return R.error("参数错误");
        }

        final HttpHeaders requestHeaders = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        requestHeaders.setContentType(type);

        //获取派单id
        Info0002 info0002 = info0002Mapper.getByServiceAutoId(receiveOrderVo.getServiceAutoId());
        if (null == info0002 || StringUtils.isBlank(info0002.getServiceAutoId())) {
            return R.error("参数错误");
        }
        JSONObject args = new JSONObject(true);
        args.put("service_auto_id", info0002.getServiceAutoId());
        args.put("order_auto_id", receiveOrderVo.getServiceAutoId());
        args.put("field_101", receiveOrderVo.getDealExplain());
        args.put("field_102", receiveOrderVo.getImageOne());
        args.put("field_103", receiveOrderVo.getImageTwo());
        args.put("field_104", receiveOrderVo.getImageThree());
        args.put("field_105", receiveOrderVo.getVoice());
        args.put("field_106", receiveOrderVo.getNotice());
        args.put("field_107", new Date());
        args.put("field_108", null);
        args.put("field_109", null);
        args.put("field_110", null);
        log.info("打印入参数据：" + args.toJSONString());
        HttpEntity<String> formEntity = new HttpEntity<>(args.toJSONString(), requestHeaders);
        ResponseEntity<JSONObject> responseEntity = this.restTemplate.postForEntity(Common.RECEIVE_ORDER_URL, formEntity, JSONObject.class);
        return R.ok(responseEntity.getBody());
    }

    @Override
    public R<PageInfo<SendOrderVo>> getPage(OrderQueryVo orderQueryVo) {
        log.info("ljfl-service[]SendOrderServiceImpl[]receiveOrder[]input.param.orderQueryVo：{}" + orderQueryVo);
        if (null == orderQueryVo || null == orderQueryVo.getSendType()) {
            return R.error("参数错误");
        }
        pageHandle(orderQueryVo);
        //获取派单总数量
        StringBuilder sql = new StringBuilder();
        Integer total;
        sql.append("select count(1) from info0001 inf1 left join info0002 inf2 on inf1.auto_id=inf2.service_auto_id ");

        //确认派单来源类型 0：远程值守派单 1督查中心派单
        if (orderQueryVo.getSendType() == 0) {
            sql.append(" where inf1.field_112 = '0'");
        } else {
            sql.append(" where inf1.field_112 = '1'");
        }

        //搜索条件参数拼接
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(orderQueryVo.getFieldName()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(orderQueryVo.getCondition()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(orderQueryVo.getContent())) {
            sql.append(" and " + orderQueryVo.getFieldName() + "");

            if (orderQueryVo.getCondition().equals("equals")) {
                sql.append("= ?");
            }

            if (orderQueryVo.getCondition().equals("like")) {
                sql.append(" LIKE CONCAT('%', ?, '%')");
            }

            if (orderQueryVo.getFieldName().equals("inf2.status")) {
                if (orderQueryVo.getContent().equals("未处理")) {
                    orderQueryVo.setContent("1");
                } else {
                    orderQueryVo.setContent("2");
                }
            }

            total = daosupport.queryForInt(sql.toString(), orderQueryVo.getContent());
        } else {
            total = daosupport.queryForInt(sql.toString(), null);
        }

        //根据分页获取派单信息
        StringBuilder hql = new StringBuilder();
        List<SendOrderVo> list;
        hql.append("select inf1.auto_id as autoId, inf1.field_105 as inspectorName, inf1.field_109 as rubbishSite, " +
                "inf1.field_106 as nickName,inf2.status as dealStatus, inf1.field_101 as reason, " +
                "inf1.field_108 as sendTime, inf2.field_107 as dealTime,inf1.field_113 as cameraNumber" +
                " from info0001 inf1 left join info0002 inf2 on inf1.auto_id=inf2.service_auto_id");

        //确认派单来源类型 0：远程值守派单 1督查中心派单
        if (orderQueryVo.getSendType() == 0) {
            hql.append(" where inf1.field_112 = 0");
        } else {
            hql.append(" where inf1.field_112 = 1");
        }

        //搜索条件参数拼接
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(orderQueryVo.getFieldName()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(orderQueryVo.getCondition()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(orderQueryVo.getContent())) {
            hql.append("  and  " + orderQueryVo.getFieldName() + "");
            if (orderQueryVo.getCondition().equals("equals")) {
                hql.append("= ?");
            }

            if (orderQueryVo.getCondition().equals("like")) {
                hql.append(" LIKE CONCAT('%',?, '%')");
            }
            hql.append(" order by inf1.field_108  limit ?,?");
            list = daosupport.queryForList(hql.toString(), SendOrderVo.class,
                    orderQueryVo.getContent(), orderQueryVo.getStartNum(),
                    orderQueryVo.getEndNum());
        } else {
            hql.append(" order by inf1.field_108 desc limit ?,?");
            list = daosupport.queryForList(hql.toString(), SendOrderVo.class,
                    orderQueryVo.getStartNum(), orderQueryVo.getEndNum());
        }

        //构造部分返回参数
        for (SendOrderVo sendOrderVo : list) {
            //把派单时间戳转为yyyy-mm-dd hh:MM:ss
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            long sendTime = Long.valueOf(sendOrderVo.getSendTime());
            Date date = new Date(sendTime);
            String time = format.format(date);
            sendOrderVo.setSendTime(time);

            //把接单处理时间戳转为yyyy-mm-dd hh:MM:ss
            if (StringUtils.isNotEmpty(sendOrderVo.getDealTime())) {
                long dealTime = Long.valueOf(sendOrderVo.getDealTime());
                Date deal_time = new Date(dealTime);
                String dtime = format.format(deal_time);
                sendOrderVo.setDealTime(dtime);
            }
            //根据接单人姓名，获取角色信息
            if (StringUtils.isNotEmpty(sendOrderVo.getNickName())) {
                AccountUsers accountUsers = accountUsersMapper.getByName(sendOrderVo.getNickName());
                if (null != accountUsers) {
                    SystemRoles role = rolesMapper.getById(accountUsers.getRoleId());
                    if (null != role) {
                        sendOrderVo.setRoleName(role.getName());
                    }
                }
            }
        }

        PageInfo<SendOrderVo> info = new PageInfo<>();
        info.setPageSize(orderQueryVo.getPageSize());
        info.setPageNum(orderQueryVo.getPageNo());
        info.setTotal(total);
        info.setList(list);
        return R.ok(info);
    }

    @Override
    public R<OrderDetailVo> getDetail(String autoId) {
        log.info("ljfl-service[]SendOrderServiceImpl[]getDetail[]input.param.autoId：{}" + autoId);
        if (StringUtils.isBlank(autoId)) {
            return R.error("参数错误");
        }

        OrderDetailVo orderDetailVo = info0001Mapper.getRetail(autoId);
        return R.ok(orderDetailVo);
    }

    @Override
    public R<OrderBlackVo> orderBlack(String autoId) {
        log.info("ljfl-service[]SendOrderServiceImpl[]orderBlack[]input.param.autoId：{}" + autoId);
        if (StringUtils.isBlank(autoId)) {
            return R.error("参数错误");
        }
        OrderBlackVo orderBlackVo = info0002Mapper.getOrderBlack(autoId);
        return R.ok(orderBlackVo);
    }

    @Override
    public R<PageInfo<ReceiveOrderResponse>> getList(ReceiveQuestVo receiveQuestVo) {
        log.info("ljfl-service[]SendOrderServiceImpl[]getDetail[]input.param.receiveQuestVo：{}" + receiveQuestVo);
        if (null == receiveQuestVo) {
            return R.error("参数错误");
        }
        pageHandle(receiveQuestVo);

        Map<String, Object> map = new HashMap<>(16);
        map.put("isFinish", receiveQuestVo.getStatus());

        //判断用户是否是超级管理员
        AccountUsers accountUsers = accountUsersMapper.getByAutoId(receiveQuestVo.getAutoId());
        if (null != accountUsers && accountUsers.getIsSupper() != 1) {
            map.put("autoId", receiveQuestVo.getAutoId());
        }

        Long total = taskMapper.getListCount(map);
        map.put("startNum", receiveQuestVo.getStartNum());
        map.put("endNum", receiveQuestVo.getEndNum());
        map.put("status", receiveQuestVo.getStatus());
        List<ReceiveOrderResponse> list = new ArrayList<>();

        //已处理列表，按修改时间降序
        if (receiveQuestVo.getStatus() == 1) {
            map.put("status", receiveQuestVo.getStatus());
            list = taskMapper.getListStatus(map);
        } else {
            list = taskMapper.getList(map);
        }

        SimpleDateFormat format=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (ReceiveOrderResponse response : list) {
            //获取派单信息
            OrderDetailVo orderDetailVo = info0001Mapper.getRetail(response.getAutoId());
            if (null != orderDetailVo && StringUtils.isNotEmpty(response.getAutoId())) {
                String title = response.getTitle();
                if (StringUtils.isNotEmpty(orderDetailVo.getSite())) {
                    title = title + orderDetailVo.getSite();
                }

                title = title + "|" + format.format(response.getCreateTime());
                response.setTitle(title);
            }
        }

        PageInfo<ReceiveOrderResponse> info = new PageInfo<>();
        info.setPageSize(receiveQuestVo.getPageSize());
        info.setPageNum(receiveQuestVo.getPageNo());
        info.setTotal(total);
        info.setList(list);
        return R.ok(info);
    }

    @Override
    public R<ReceiveOrderDetailVo> getReceiveDetail(String autoId) {
        log.info("ljfl-service[]SendOrderServiceImpl[]getReceiveDetail[]input.param.autoId：{}" + autoId);
        if (StringUtils.isBlank(autoId)) {
            return R.error("参数错误");
        }

        ReceiveOrderDetailVo receiveVo = info0002Mapper.getReceiveDetail(autoId);
        return R.ok(receiveVo);
    }

    //扫描将要报名结束的数据
    private void pageHandle(OrderQueryVo orderQueryVo) {
        Integer pageNum = orderQueryVo.getPageNo();
        Integer pageSize = orderQueryVo.getPageSize();
        if (null == pageSize || pageSize == 0) pageSize = 10;
        if (null == pageNum || pageNum == 0) pageNum = 1;
        Integer endNum = pageSize;
        Integer startNum = (pageNum - CommonAttributes.NUM_ONE) * pageSize;
        orderQueryVo.setEndNum(endNum);
        orderQueryVo.setStartNum(startNum);
        orderQueryVo.setPageNo(pageNum);
        orderQueryVo.setPageSize(pageSize);
    }

    private void pageHandle(ReceiveQuestVo receiveQuestVo) {
        Integer pageNum = receiveQuestVo.getPageNo();
        Integer pageSize = receiveQuestVo.getPageSize();
        if (null == pageSize || pageSize == 0) pageSize = 10;
        if (null == pageNum || pageNum == 0) pageNum = 1;
        Integer endNum = pageSize;
        Integer startNum = (pageNum - CommonAttributes.NUM_ONE) * pageSize;
        receiveQuestVo.setEndNum(endNum);
        receiveQuestVo.setStartNum(startNum);
        receiveQuestVo.setPageNo(pageNum);
        receiveQuestVo.setPageSize(pageSize);
    }
}
