package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.*;
import cn.wisenergy.model.dto.*;
import cn.wisenergy.model.vo.CameraIndexVo;
import cn.wisenergy.service.ICameraApiService;
import cn.wisenergy.service.app.IFfmpegBackRecordService;
import cn.wisenergy.service.app.ISiteWatchOverService;
import com.alibaba.excel.util.CollectionUtils;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotBlank;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;


/**
 * 站点值守
 */
@Service
@Slf4j
public class SiteWatchOverServiceImpl implements ISiteWatchOverService {
    @Autowired
    private RubbishMapper rubbishMapper;

    @Autowired
    private AccountOrganizationMapper accountOrganizationMapper;

    @Autowired
    private CameraMapper cameraMapper;

    @Autowired
    ICameraApiService cameraApiService;

    @Autowired
    private AccountUsersMapper usersMapper;

    @Autowired
    IFfmpegBackRecordService ffmpegBackRecordService;

    @Override
    public R pageVideo(VideoPageQry qry) {
        CameraPage cameraPage = new CameraPage();
        cameraPage.setPageNo(1);
        cameraPage.setPageSize(1000);
        R<Object> result = cameraApiService.cameras(cameraPage);
        CameraIndexResultDto cameraIndexResultDto = null;
        if (0 == result.getCode()) {
            Object data = result.getData();
            String string = JSON.toJSONString(data);
            cameraIndexResultDto = JSON.parseObject(string, CameraIndexResultDto.class);
        }
        //  监控点唯一标识
        List<CameraMonitory> cameraIndexCodes = new ArrayList<>();

        if (cameraIndexResultDto != null) {
            List<CameraIndexSubDto> list = cameraIndexResultDto.getList();
            for (int i = 0; i < list.size(); i++) {
                CameraMonitory cameraMonitory = new CameraMonitory();
                cameraMonitory.setCameraId(i + "");

                cameraMonitory.setCameraIndexCode(list.get(i).getCameraIndexCode());
                cameraIndexCodes.add(cameraMonitory);
            }
        }

//        //todo  根据类型分别查询各种关联综合安防管理平台的摄像机  （监控点）
//        //写死一个监控点
//        CameraMonitory cameraMonitory = new CameraMonitory();
//        cameraMonitory.setCameraIndexCode("8cce4ab74fda48cab17166e8f2f04310");
//        cameraMonitory.setCameraId("1");
////        cameraIndexCodes.add("72aba1a2ce484f84bc32468a828e82ab");
//        cameraIndexCodes.add(cameraMonitory);

        return R.ok(cameraIndexCodes);
    }

    @Override
    public List<CameraMonitory> getCameraMonitory(VideoPageQry qry) {
        CameraPage cameraPage = new CameraPage();
        cameraPage.setPageNo(1);
        cameraPage.setPageSize(1000);
        R<Object> result = cameraApiService.cameras(cameraPage);
        CameraIndexResultDto cameraIndexResultDto = null;
        if (0 == result.getCode()) {
            Object data = result.getData();
            String string = JSON.toJSONString(data);
            cameraIndexResultDto = JSON.parseObject(string, CameraIndexResultDto.class);
        }
        //  监控点唯一标识
        List<CameraMonitory> cameraIndexCodes = new ArrayList<>();

        if (cameraIndexResultDto != null) {
            List<CameraIndexSubDto> list = cameraIndexResultDto.getList();
            for (int i = 0; i < list.size(); i++) {
                CameraMonitory cameraMonitory = new CameraMonitory();
                cameraMonitory.setCameraId(i + "");

                cameraMonitory.setCameraIndexCode(list.get(i).getCameraIndexCode());
                cameraIndexCodes.add(cameraMonitory);
            }
        }
        return cameraIndexCodes;
    }

    /**
     * 根据监控点查询视频预览URL
     *
     * @param previewUrls
     * @return
     */
    @Override
    public R videoUrls(CamerasPreviewUrls previewUrls) {
        List<CameraVideoUrlsDto> data = new ArrayList<>();
        //复制属性
        CameraPreviewUrls cameraPreviewUrls = new CameraPreviewUrls();
        BeanUtils.copyProperties(previewUrls, cameraPreviewUrls);
        //
        List<String> cameraIndexCodes = previewUrls.getCameraIndexCodes();
        if (CollectionUtils.isEmpty(cameraIndexCodes)) {
            return R.ok();
        }
        //循环查询视频URL
        for (String cameraIndexCode : cameraIndexCodes) {
            cameraPreviewUrls.setCameraIndexCode(cameraIndexCode);
            R<Object> objectR = cameraApiService.camerasPreviewUrls(cameraPreviewUrls);
            //
            CameraVideoUrlsDto cameraVideoUrlsDto = new CameraVideoUrlsDto();
            cameraVideoUrlsDto.setCameraIndexCode(cameraIndexCode);
            if (objectR.getCode() == 0) {      //成功
                cameraVideoUrlsDto.setResult(objectR.getData());
            }

            data.add(cameraVideoUrlsDto);
        }
        return R.ok(data);
    }

    /**
     * 根据一个监控点查询视频预览Urls
     *
     * @param previewUrl
     * @return
     */
    @Override
    public R videoUrl(CameraPreviewUrls previewUrl) {
        //判断是否为空
        if (StringUtils.isBlank(previewUrl.getCameraIndexCode())) {
            return R.ok();
        }
        //
        R<Object> objectR = cameraApiService.camerasPreviewUrls(previewUrl);
        return objectR;
    }

    /**
     * 根据一个监控点查询录像回放Urls
     *
     * @param playbackURLs
     * @return
     */
    @Override
    public R playbackUrl(CameraPlaybackURLs playbackURLs) {
        //判断是否为空
        if (StringUtils.isBlank(playbackURLs.getCameraIndexCode())) {
            return R.ok();
        }
        //
        R result = cameraApiService.playbackURLs(playbackURLs);
        return result;
    }

    /**
     * 根据多个监控点查询录像回放Urls
     *
     * @param playbackURLs
     * @return
     */
    @Override
    public R playbackUrls(CameraPlaybacksURLs playbackURLs) {
        List<CameraVideoUrlsDto> data = new ArrayList<>();
        //参数
        List<String> cameraIndexCodes = playbackURLs.getCameraIndexCodes();
        if (CollectionUtils.isEmpty(cameraIndexCodes)) {
            return R.ok();
        }
        //复制属性
        CameraPlaybackURLs cameraPlaybackURLs = new CameraPlaybackURLs();
        BeanUtils.copyProperties(playbackURLs, cameraPlaybackURLs);
        //循环查询视频URL
        for (String cameraIndexCode : cameraIndexCodes) {
            cameraPlaybackURLs.setCameraIndexCode(cameraIndexCode);
            R<Object> objectR = cameraApiService.playbackURLs(cameraPlaybackURLs);
            //
            CameraVideoUrlsDto cameraVideoUrlsDto = new CameraVideoUrlsDto();
            cameraVideoUrlsDto.setCameraIndexCode(cameraIndexCode);
            if (objectR.getCode() == 0) {      //成功
                cameraVideoUrlsDto.setResult(objectR.getData());
            }

            data.add(cameraVideoUrlsDto);
        }
        return R.ok(data);
    }

    /**
     * 根据一个监控点进行手动抓图
     *
     * @param cameraIndexCode
     * @return
     */
    @Override
    public R manualCapture(CameraIndexCode cameraIndexCode) {
        //
        if (StringUtils.isBlank(cameraIndexCode.getCameraIndexCode())) {
            return R.ok();
        }
        //
        R manualCapture = cameraApiService.manualCapture(cameraIndexCode);
        log.info("站点值守,手动抓图返回结果：" + manualCapture);
        return manualCapture;
    }

    /**
     * 根据一个监控点进行录像锁定与解锁
     *
     * @param recordLock
     * @return
     */
    @Override
    public R recordLock(CameraRecordLock recordLock) {
        R result = cameraApiService.recordLock(recordLock);
        return result;
    }

    /**
     * 根据一个监控点进行开始录像操作
     *
     * @param recordStart
     * @return
     */
    @Override
    public R recordStart(CameraManualRecordStart recordStart) {

        R result = cameraApiService.manualRecordStart(recordStart);
        return result;
    }


    @Override
    public R recordStop(CameraManualRecordStop recordStop) {

        R result = cameraApiService.manualRecordStop(recordStop);
        return result;
    }

    /**
     * 根据一个监控点进行手动录像状态获取操作
     *
     * @param recordStatus
     * @return
     */
    @Override
    public R recordStatus(CameraManualRecordStop recordStatus) {
        R result = cameraApiService.manualRecordStatus(recordStatus);
        return result;
    }

    /**
     * 根据一个监控点查询手动录像编号
     *
     * @param recordSearch
     * @return
     */
    @Override
    public R recordSearch(CameraIndexCodes recordSearch) {
        R result = cameraApiService.manualRecordTaskIdSearch(recordSearch);
        return result;
    }

    /**
     * 根据条件分页参数获取监控点在线状态列表
     *
     * @param onlineStatus
     * @return
     */
    @Override
    public R onlineStatus(CameraOnlineStatus onlineStatus) {
        R result = cameraApiService.onlineCamera(onlineStatus);
        return result;
    }

    /**
     * 分页获取安防平台监控点列表
     *
     * @param cameraPage
     * @return
     */
    @Override
    public R pageCameraIndex(CameraPage cameraPage) {
        R<Object> cameras = cameraApiService.cameras(cameraPage);
        return cameras;
    }

    @Override
    public R<List<CameraIndexVo>> getByAutoId(String autoId, String userAutoId) {
        if (StringUtils.isBlank(userAutoId) || StringUtils.isBlank(autoId)) {
            return R.error("入参为空！");
        }
        //初始化返回参数
        List<CameraIndexVo> cameraIndexVos = new ArrayList<>();

        //获取登录用户信息
        AccountUsers accountUsers = usersMapper.getByAutoId(userAutoId);

        //获取顶级组织 TODO 后续多个区需要改代码
        List<String> org = accountOrganizationMapper.getAreaId();
        if (autoId.equals(org.get(0)) && accountUsers.getIsSupper() == 1) {
            VideoPageQry qry = new VideoPageQry();
            List<CameraMonitory> cList = getCameraMonitory(qry);
            if (CollectionUtils.isEmpty(cList)) {
                return R.ok(cameraIndexVos);
            }

            //根据摄像头code,获取垃圾站ids
            List<String> stringList = cList.stream().map(CameraMonitory::getCameraIndexCode).collect(Collectors.toList());
            List<Camera> cameraList = cameraMapper.getByNumber(stringList);
            if (CollectionUtils.isEmpty(cameraList)) {
                return R.ok(cameraIndexVos);
            }

            //构造返回数据
            for (Camera camera : cameraList) {
                //获取垃圾站信息
                CameraIndexVo indexVo = new CameraIndexVo();
                Rubbish rubbish = rubbishMapper.getById(camera.getRubbishId());
                if (null != rubbish) {
                    indexVo.setLiveTime(rubbish.getLiveTime());
                    indexVo.setClickTime(rubbish.getClickTime());
                    indexVo.setCameraCode(camera.getCameraNumber());
                    cameraIndexVos.add(indexVo);
                }
            }
            return R.ok(cameraIndexVos);
        }

        if (StringUtils.isBlank(autoId)) {
            return R.error("入参为空！");
        }

        //1、获取社区信息
        AccountOrganization organization = null;
        if (accountUsers.getIsSupper() == 1) {
            organization = accountOrganizationMapper.getAreaByAutoId(autoId);
            if (null == organization) {
                return R.error("社区信息不存在！");
            }
        } else {
            organization = accountOrganizationMapper.getAreaByAutoId(accountUsers.getOrganizationAutoId());
            if (null == organization) {
                return R.error("社区信息不存在！");
            }
        }


        //2、是区
        List<String> name = new ArrayList<>();
        if (organization.getParentId().equals("NONE") && organization.getRootId().equals("NONE")) {
            name = accountOrganizationMapper.getByRootId(organization.getAutoId());
        } else if (organization.getRootId().equals(organization.getParentId()) && !organization.getRootId().equals("NONE")) {
            //3、是街道
            name = accountOrganizationMapper.getByParentId(organization.getAutoId());
        } else {
            //4、社区
            name.add(organization.getName());
        }

        //根据社区信息，获取垃圾站信息
        List<Rubbish> rubbishList = rubbishMapper.getByName(name);
        if (CollectionUtils.isEmpty(rubbishList)) {
            return R.error("垃圾站不存在！");
        }

        for (Rubbish rubbish : rubbishList) {
            //根据垃圾站信息,获取摄像头编码
            List<Camera> cameras = cameraMapper.getByRubbishId(rubbish.getId());
            if (!CollectionUtils.isEmpty(cameras)) {
                for (Camera camera : cameras) {
                    CameraIndexVo indexVo = new CameraIndexVo();
                    indexVo.setLiveTime(rubbish.getLiveTime());
                    indexVo.setClickTime(rubbish.getClickTime());
                    indexVo.setCameraCode(camera.getCameraNumber());
                    cameraIndexVos.add(indexVo);
                }
            }
        }

        return R.ok(cameraIndexVos);
    }

    /**
     * 通过社区查询垃圾站下的摄像头回放URLs，带分页
     *
     * @param
     * @param cameraListDto
     * @return
     */
    @Override
    public R pageCameraBackUrls(CameraListDto cameraListDto) {
        String userAutoId = cameraListDto.getUserAutoId();
        String autoId = cameraListDto.getAutoId();

        //初始化返回参数
        Page<FfmpegBackRecord> page = new Page<>(cameraListDto.getPageNo(), cameraListDto.getPageSize(), true);

        //获取登录用户信息
        AccountUsers accountUsers = usersMapper.getByAutoId(userAutoId);

        //获取顶级组织 TODO 后续多个区需要改代码
        List<String> org = accountOrganizationMapper.getAreaId();

        //查询垃圾站的条件
        QueryWrapper<Rubbish> rubbishQueryWrapper = new QueryWrapper<>();

        //超级管理员  查询全部摄像机的回放
        if (autoId.equals(org.get(0)) && accountUsers.getIsSupper() == 1) {
            rubbishQueryWrapper.eq("is_delete", 0);
        } else {
            //不是管理员
            //1、获取社区信息
            AccountOrganization organization = null;

            if (accountUsers.getIsSupper() == 1) {
                organization = accountOrganizationMapper.getAreaByAutoId(autoId);
                if (null == organization) {
                    return R.error("社区信息不存在！");
                }
            } else {
                organization = accountOrganizationMapper.getAreaByAutoId(accountUsers.getOrganizationAutoId());
                if (null == organization) {
                    return R.error("社区信息不存在！");
                }
            }

            List<String> name = new ArrayList<>();
            //2、是区
            if (organization.getParentId().equals("NONE") && organization.getRootId().equals("NONE")) {
                name = accountOrganizationMapper.getByRootId(organization.getAutoId());

            } else if (organization.getRootId().equals(organization.getParentId()) && !organization.getRootId().equals("NONE")) {
                //3、是街道
                name = accountOrganizationMapper.getByParentId(organization.getAutoId());
            } else {
                //4、社区
                name.add(organization.getName());
            }

            //不是管理员的条件
            rubbishQueryWrapper.eq("is_delete", 0)
                    .in("community", name);
        }

        //根据社区信息，获取垃圾站信息
        List<Rubbish> rubbishList = rubbishMapper.selectList(rubbishQueryWrapper);
        if (CollectionUtils.isEmpty(rubbishList)) {
            return R.error("垃圾站不存在！");
        }

        //根据垃圾站查询摄像机
        List<Integer> rubbishIds = rubbishList.stream().map(Rubbish::getId).collect(Collectors.toList());
        QueryWrapper<Camera> cameraQueryWrapper = new QueryWrapper<>();
        cameraQueryWrapper.eq("is_delete", 0)
                .in("rubbish_id", rubbishIds);
        List<Camera> cameras = cameraMapper.selectList(cameraQueryWrapper);
        if (CollectionUtils.isEmpty(cameras)) {
            return R.ok(page);
        }
        List<Integer> cameraIds = cameras.stream().map(Camera::getId).collect(Collectors.toList());
        //分页查询摄像机的回放URL
        QueryWrapper<FfmpegBackRecord> backRecordQueryWrapper = new QueryWrapper<>();
        backRecordQueryWrapper.in("camear_id", cameraIds)
                .ge("begin_time", cameraListDto.getBeginTime())
                .le("end_time", cameraListDto.getEndTime())
                .eq("status", 0)  //加入0 取正常的连接
                .orderByDesc("begin_time");
        IPage<FfmpegBackRecord> recordIPage = ffmpegBackRecordService.page(page, backRecordQueryWrapper);
        //设置cameraNum
        List<FfmpegBackRecord> records = recordIPage.getRecords();
        records.stream().forEach(r -> {
            cameras.stream().forEach(c -> {
                if (r.getCamearId() != null && r.getCamearId().equals(c.getId())) {
                    r.setCameraNumber(c.getCameraNumber());
                }
            });
        });
        return R.ok(recordIPage);
    }
}
