package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.constant.CommonAttributes;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.AccountOrganizationMapper;
import cn.wisenergy.mapper.AccountUsersMapper;
import cn.wisenergy.mapper.Info0001Mapper;
import cn.wisenergy.mapper.RubbishMapper;
import cn.wisenergy.model.app.AccountOrganization;
import cn.wisenergy.model.app.AccountUsers;
import cn.wisenergy.model.vo.StreetQueryVo;
import cn.wisenergy.model.vo.StreetShowVo;
import cn.wisenergy.model.vo.StreetVo;
import cn.wisenergy.service.app.Daosupport;
import cn.wisenergy.service.app.StreetService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class StreetServiceImpl extends ServiceImpl<AccountOrganizationMapper, AccountOrganization> implements StreetService {
    @Autowired
    private AccountOrganizationMapper organizationMapper;

    @Autowired
    private RubbishMapper rubbishMapper;

    @Autowired
    private Daosupport daosupport;

    @Autowired
    private Info0001Mapper info0001Mapper;

    @Autowired
    private AccountUsersMapper usersMapper;

    @Override
    public R<PageInfo<StreetShowVo>> getList(StreetQueryVo streetQueryVo) {
        log.info("ljfl-service[]StreetServiceImpl[]getList[]input.param.streetQueryVo:{}" + streetQueryVo);
        if (null == streetQueryVo) {
            return R.error("入参为空");
        }
        pageHandle(streetQueryVo);
        //1、获取街道总数
        StringBuilder sql = new StringBuilder();
        Integer total;
        sql.append("select count(1) from account_organization where type = 6 and parent_id != 'NONE' and root_id != 'NONE' and parent_id != root_id");//and parent_id != root_id
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(streetQueryVo.getFieldName()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(streetQueryVo.getCondition()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(streetQueryVo.getContent())) {
            sql.append(" and  " + streetQueryVo.getFieldName() + "");
            if (streetQueryVo.getCondition().equals("equals")) {
                sql.append("= ?");
            }

            if (streetQueryVo.getCondition().equals("like")) {
                sql.append(" LIKE CONCAT('%', ? , '%')");
            }
            log.info("打印sql:" + sql.toString());
            total = daosupport.queryForInt(sql.toString(), streetQueryVo.getContent());
        } else {
            //统计数据总条数
            total = daosupport.queryForInt(sql.toString(), null);
        }

        StringBuilder hql = new StringBuilder();
        List<StreetShowVo> list;
        hql.append("select auto_id as autoId,parent_id as parentId,root_id as rootId " +
                "from account_organization where type = 6 and parent_id != 'NONE' and root_id != 'NONE' and parent_id != root_id "); //and parent_id != root_id
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(streetQueryVo.getFieldName()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(streetQueryVo.getCondition()) &&
                org.apache.commons.lang3.StringUtils.isNotEmpty(streetQueryVo.getContent())) {
            hql.append(" and " + streetQueryVo.getFieldName() + "");
            if (streetQueryVo.getCondition().equals("equals")) {
                hql.append("= ?");
            }

            if (streetQueryVo.getCondition().equals("like")) {
                hql.append(" LIKE CONCAT('%', ?, '%')");
            }
            hql.append(" limit ?,?");
            list = daosupport.queryForList(hql.toString(), StreetShowVo.class, streetQueryVo.getContent(),
                    streetQueryVo.getStartNum(), streetQueryVo.getEndNum());
        } else {
            hql.append(" limit ?,?");
            list = daosupport.queryForList(hql.toString(), StreetShowVo.class,
                    streetQueryVo.getStartNum(), streetQueryVo.getEndNum());
        }
        for (StreetShowVo streetShowVo : list) {

            //获取社区
            AccountOrganization account = organizationMapper.getAreaByAutoId(streetShowVo.getAutoId());
            if (null != account) {
                streetShowVo.setCommunityName(account.getName());
                streetShowVo.setCreateTime(account.getCreateTime());
            }

            //获取街道
            AccountOrganization street = organizationMapper.getAreaByAutoId(streetShowVo.getParentId());
            if (null != street) {
                streetShowVo.setStreetName(street.getName());
            }

            //获取区县
            AccountOrganization area = organizationMapper.getAreaByAutoId(streetShowVo.getRootId());
            if (null != area) {
                streetShowVo.setAreaName(area.getName());
            }

            //获取垃圾站数量
            int count = rubbishMapper.countSum(streetShowVo.getStreetName(), streetShowVo.getCommunityName());
            streetShowVo.setRubbishCount(count);
        }

        PageInfo<StreetShowVo> info = new PageInfo<>();
        info.setPageSize(streetQueryVo.getPageSize());
        info.setPageNum(streetQueryVo.getPageNo());
        info.setTotal(total);
        info.setList(list);
        return R.ok(info);
    }

    @Override
    public R<List<StreetVo>> getTree() {
        log.info("ljfl-serviceStreetServiceImpl[]getTree[]input.method");
        //获取所有街道
        List<StreetVo> rootList = organizationMapper.getStreetList();
        if (CollectionUtils.isEmpty(rootList)) {
            return R.ok(new ArrayList<>());
        }

        //构造树形结构
        List<StreetVo> streetList = getStreetList(rootList);

        //结果树
        List<StreetVo> result = new ArrayList<>();
        result.addAll(streetList);
        return R.ok(result);
    }

    @Override
    public R<List<StreetVo>> getAllStreet() {
        log.info("ljfl-service[]StreetServiceImpl[]getAllStreet[]input.method.getRubbishNameList");
        //1、获取区县的autoId
        List<String> autoIds = organizationMapper.getAreaId();

        //2、根据区县信息获取街道列表
        List<StreetVo> list = organizationMapper.getStreetByAreaIds(autoIds);
        return R.ok(list);
    }

    @Override
    public R<List<StreetVo>> getByStreetId(String autoId) {
        log.info("ljfl-service[]StreetServiceImpl[]getList[]input.param.autoId:{}" + autoId);
        if (StringUtils.isEmpty(autoId)) {
            return R.error("入参为空");
        }
        List<StreetVo> list = organizationMapper.getByStreetId(autoId);
        return R.ok(list);
    }

    @Override
    public R<List<StreetVo>> getUserTree(String autoId) {
        log.info("ljfl-service[]StreetServiceImpl[]getUserTree[]input.param.autoId:{}" + autoId);
        if (StringUtils.isEmpty(autoId)) {
            return R.error("入参为空");
        }
        //获取当前登录用户信息
        AccountUsers accountUsers = usersMapper.getByAutoId(autoId);
        if (null == accountUsers) {
            return R.error("登录用户信息不存在！");
        }

        if (accountUsers.getIsSupper() == 1) {
            return getTree();
        }

        if (StringUtils.isEmpty(accountUsers.getOrganizationAutoId())) {
            return R.error("当前用户没有关联街道信息！");
        }

        //获取组织信息
        AccountOrganization ao = organizationMapper.getAreaByAutoId(accountUsers.getOrganizationAutoId());
        List<StreetVo> result = new ArrayList<>();
        StreetVo streetVo = new StreetVo();
        if (ao.getParentId().equals(ao.getRootId()) && !ao.getParentId().equals("NONE")) {
            //获取父级组织信息
            AccountOrganization aot = organizationMapper.getAreaByAutoId(ao.getParentId());
            streetVo.setAutoId(aot.getAutoId());
            streetVo.setName(aot.getName());
            streetVo.setParentId(aot.getParentId());
            streetVo.setRootId(aot.getRootId());

            List<StreetVo> list = new ArrayList<>();
            StreetVo children = new StreetVo();
            children.setAutoId(ao.getAutoId());
            children.setParentId(ao.getParentId());
            children.setRootId(ao.getRootId());
            children.setName(ao.getName());
            list.add(children);
            streetVo.setChildren(list);
            result.add(streetVo);
        }
        if (!ao.getParentId().equals(ao.getRootId()) && !ao.getParentId().equals("NONE") && !ao.getRootId().equals("NONE")) {
            //获取父级组织信息
            AccountOrganization aot = organizationMapper.getAreaByAutoId(ao.getParentId());

            //获取顶级组织信息
            AccountOrganization root = organizationMapper.getAreaByAutoId(ao.getRootId());

            //顶级
            streetVo.setAutoId(root.getAutoId());
            streetVo.setName(root.getName());
            streetVo.setRootId(root.getRootId());

            //父级
            List<StreetVo> list = new ArrayList<>();
            StreetVo children = new StreetVo();
            children.setAutoId(aot.getAutoId());
            children.setParentId(aot.getParentId());
            children.setRootId(aot.getRootId());
            children.setName(aot.getName());
            list.add(children);

            //本级
            List<StreetVo> sunList = new ArrayList<>();
            StreetVo sun = new StreetVo();
            sun.setAutoId(ao.getAutoId());
            sun.setParentId(ao.getParentId());
            sun.setRootId(ao.getRootId());
            sun.setName(ao.getName());
            sunList.add(sun);

            streetVo.setChildren(list);
            children.setChildren(sunList);
            result.add(streetVo);

        }
        return R.ok(result);
    }

    //扫描将要报名结束的数据
    private void pageHandle(StreetQueryVo streetQueryVo) {
        Integer pageNum = streetQueryVo.getPageNo();
        Integer pageSize = streetQueryVo.getPageSize();
        if (null == pageSize || pageSize == 0) pageSize = 10;
        if (null == pageNum || pageNum == 0) pageNum = 1;
        Integer endNum = pageSize;
        Integer startNum = (pageNum - CommonAttributes.NUM_ONE) * pageSize;
        streetQueryVo.setEndNum(endNum);
        streetQueryVo.setStartNum(startNum);
        streetQueryVo.setPageNo(pageNum);
        streetQueryVo.setPageSize(pageSize);
    }

    private List<StreetVo> getStreetList(List<StreetVo> list) {
        List<StreetVo> rootList = new ArrayList<>();
        List<StreetVo> childrenList = new ArrayList<>();

        //1|遍历出根节点和子节点
        for (StreetVo streetVo : list) {
            if (streetVo.getRootId().equals("NONE")) {
                rootList.add(streetVo);
            } else {
                childrenList.add(streetVo);
            }
        }

        //2.遍历集合，获取根节点下的子节点
        for (StreetVo streetVo : rootList) {
            List<StreetVo> children = setChildren(streetVo.getAutoId(), childrenList);
            streetVo.setChildren(children);
        }
        return rootList;
    }

    private List<StreetVo> setChildren(String id, List<StreetVo> list) {
        List<StreetVo> children = new ArrayList<>();
        for (StreetVo streetVo : list) {
            if (id.equals(streetVo.getParentId())) {
                children.add(streetVo);
            }
        }

        for (int i = 0; i < children.size(); i++) {
            String autoId = children.get(i).getAutoId();
            List<StreetVo> childrenList = setChildren(autoId, list);
            children.get(i).setChildren(childrenList);
        }
        return children;
    }
}
