package cn.wisenergy.service.ffmpeg.hkws.handler;

import cn.wisenergy.service.app.IFfmpegBackRecordService;
import cn.wisenergy.service.ffmpeg.hkws.CommandManager;
import lombok.extern.slf4j.Slf4j;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * 任务消息输出处理器
 *
 * @author eguid
 * @version 2017年10月13日
 * @since jdk1.7
 */
@Slf4j
public class OutHandler extends Thread {
    /**
     * 控制状态
     */
    private volatile boolean desstatus = true;

    /**
     * 读取输出流
     */
    private BufferedReader br = null;

    /**
     * 任务ID
     */
    private String id = null;

    /**
     * 消息处理方法
     */
    private OutHandlerMethod ohm;

    /**
     * 统计日志输出的行数  大于30行时，认为推送成功
     */
    private int count;

    /**
     * spingbean 中获取 一个bean对象
     */
    IFfmpegBackRecordService ffmpegBackRecordService = SpringBeanUtil.getBean(IFfmpegBackRecordService.class);

    /**
     * 创建输出线程（默认立即开启线程）
     *
     * @param is
     * @param id
     * @param ohm
     * @return
     */
    public static OutHandler create(InputStream is, String id, OutHandlerMethod ohm) {
        return create(is, id, ohm, true);
    }

    /**
     * 创建输出线程
     *
     * @param is
     * @param id
     * @param ohm
     * @param start-是否立即开启线程
     * @return
     */
    public static OutHandler create(InputStream is, String id, OutHandlerMethod ohm, boolean start) {
        OutHandler out = new OutHandler(is, id, ohm);
        if (start)
            out.start();
        return out;
    }

    public void setOhm(OutHandlerMethod ohm) {
        this.ohm = ohm;
    }

    public void setDesStatus(boolean desStatus) {
        this.desstatus = desStatus;
    }

    public void setId(String id) {
        this.id = id;
    }

    public OutHandlerMethod getOhm() {
        return ohm;
    }

    public OutHandler(InputStream is, String id, OutHandlerMethod ohm) {
        br = new BufferedReader(new InputStreamReader(is));
        this.id = id;
        this.ohm = ohm;
        this.count = 0;
    }

    /**
     * 重写线程销毁方法，安全的关闭线程
     */
    @Override
    public void destroy() {
        setDesStatus(false);
    }

    /**
     * 执行输出线程
     */
    @Override
    public void run() {
        String msg = null;
        try {
            if (CommandManager.config.isDebug()) {
                log.info(id + "开始推流！");
                System.out.println(id + "开始推流！");
            }
            while (desstatus && (msg = br.readLine()) != null) {
                count++;
                ohm.parse(id, msg);

//                //如果消息包含 其中的某个信息 就认为此次的推送任务失败
//                if (msg.contains("Invalid data found when processing input")) {
//                    System.out.println("ffmpegBackRecordService  的 bean 是：" + ffmpegBackRecordService);
//                    System.out.println("解析到的错误行是： "+msg);
//                    ffmpegBackRecordService.changeStatusByChannelName(id);
//                }
//                //如果消息  超时  也认为失败
//                if (msg.contains("Connection timed out")){
//                    System.out.println("ffmpegBackRecordService  的 bean 是：" + ffmpegBackRecordService);
//                    System.out.println("解析到的错误行是： "+msg);
//                    ffmpegBackRecordService.changeStatusByChannelName(id);
//                }

                //输出消息大于30行   认为本次推送成功，有数据能访问
                if (count >= 30) {
                    ffmpegBackRecordService.changeStatusByChannelName(id, 0);
                    desstatus = false;    //设置为false  取消继续循环执行
                    System.err.println("本次推送日志已经达到30行，推送成功！结束日志输出线程==============================");
                }
                if (ohm.isbroken()) {
                    log.info("检测到中断，提交重启任务给保活处理器" + id);
                    System.err.println("检测到中断，提交重启任务给保活处理器");
                    //如果发生异常中断，立即进行保活
                    //把中断的任务交给保活处理器进行进一步处理
//					KeepAliveHandler.add(id);
                }
            }
        } catch (IOException e) {
            log.info("发生内部异常错误，自动关闭[" + this.getId() + "]线程");
            System.out.println("发生内部异常错误，自动关闭[" + this.getId() + "]线程");
            destroy();
        } finally {
            if (this.isAlive()) {
                destroy();
            }
        }
    }

}
