package cn.wisenergy.service.impl;

import cn.wisenergy.common.camera.CameraConfig;
import cn.wisenergy.common.constant.IscCameraConstants;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.dto.*;
import cn.wisenergy.service.ICameraApiService;
import com.alibaba.fastjson.JSON;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * ISC平台 摄像头调用
 */
@Service
@Slf4j
public class CameraApiServiceImpl implements ICameraApiService {

    @Autowired
    CameraConfig cameraConfig;


    /**
     * 获取监控点预览取流URLv2
     *
     * @param param
     * @return
     */
    @Override
    public R<Object> camerasPreviewUrls(CameraPreviewUrls param) {

        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.previewURLs;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        Map map = null;
        try {
            String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
            map = JSON.parseObject(result, Map.class);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("获取监控点预览取流URLv2失败：{}", e.getMessage());
            return R.error(e.getMessage());
        }
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 分页获取监控点资源
     *
     * @param param
     * @return
     */
    @Override
    public R<Object> cameras(CameraPage param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.cameras;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        Map map = null;
        try {
            String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
            map = JSON.parseObject(result, Map.class);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("分页获取监控点资源：{}", e.getMessage());
            return R.error(e.getMessage());
        }
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }


    /**
     * 获取监控点回放取流URLv2
     *
     * @param param
     * @return
     */
    @Override
    public R playbackURLs(CameraPlaybackURLs param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.playbackURLs;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        Map map = null;
        try {
            String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
            map = JSON.parseObject(result, Map.class);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("获取监控点回放取流URLv2：{}", e.getMessage());
            return R.error(e.getMessage());
        }
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 语音对讲  查询对讲URL
     *
     * @param param
     * @return
     */
    @Override
    public R camerasTalkURLs(CameraTalkURLs param) {
//STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.talkURLs;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        Map map = null;
        try {
            String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
            map = JSON.parseObject(result, Map.class);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("语音对讲  查询对讲URL：{}", e.getMessage());
            return R.error(e.getMessage());
        }
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 抓图 手动抓图
     *
     * @param param 监控点唯一标识，可通过分页获取监控点资源获取
     * @return
     */
    @Override
    public R manualCapture(CameraIndexCode param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.manualCapture;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        Map map = null;
        try {
            String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
            map = JSON.parseObject(result, Map.class);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("抓图 手动抓图：{}", e.getMessage());
            return R.error(e.getMessage());
        }
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 手动录像 录像锁定与解锁
     *
     * @param param
     * @return
     */
    @Override
    public R recordLock(CameraRecordLock param) {

        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.lock;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        Map map = null;
        try {
            String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
            map = JSON.parseObject(result, Map.class);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("手动录像 录像锁定与解锁：{}", e.getMessage());
            return R.error(e.getMessage());
        }
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 手动录像 开始手动录像
     *
     * @param param
     * @return
     */
    @Override
    public R manualRecordStart(CameraManualRecordStart param) {

        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.manualRecordStart;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        Map map = null;
        try {
            String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
            map = JSON.parseObject(result, Map.class);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("手动录像 录像锁定与解锁：{}", e.getMessage());
            return R.error(e.getMessage());
        }
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 手动录像 停止手动录像
     *
     * @param param
     * @return
     */
    @Override
    public R manualRecordStop(CameraManualRecordStop param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.manualRecordStop;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
        Map map = JSON.parseObject(result, Map.class);
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 手动录像 获取手动录像状态
     *
     * @param param
     * @return
     */
    @Override
    public R manualRecordStatus(CameraManualRecordStop param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.manualRecordStatus;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
        Map map = JSON.parseObject(result, Map.class);
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 手动录像 查询手动录像编号
     *
     * @param param
     * @return
     */
    @Override
    public R manualRecordTaskIdSearch(CameraIndexCodes param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.manualRecordSearch;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
        Map map = JSON.parseObject(result, Map.class);
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 根据区域编号获取下一级区域列表v2，下级区域只包括直接下级子区域。
     *
     * @param param
     * @return
     */
    @Override
    public R regionsSubRegions(CameraRegionsSubRegions param) {
        //资源类型为区域，查询区域资源
        param.setResourceType("region");

        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.regionsSubRegions;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
        Map map = JSON.parseObject(result, Map.class);
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 获取根区域信息
     *
     * @return
     */
    @Override
    public R regionsRoot() {

        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.regionsRoot;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
//        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        String result = ArtemisHttpUtil.doPostStringArtemis(path, null, null, null, contentType, null);
        Map map = JSON.parseObject(result, Map.class);
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 查询区域列表v2
     *
     * @param param
     * @return
     */
    @Override
    public R regionNodesByParams(CameraRegionNodes param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.regionNodes;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
        Map map = JSON.parseObject(result, Map.class);
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 根据区域编号获取下级监控点列表
     *
     * @param param
     * @return
     */
    @Override
    public R regionIndexCodeCameras(CamerasRegionIndexCode param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.regionIndexCodeCameras;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
        Map map = JSON.parseObject(result, Map.class);
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 获取监控点在线状态
     *
     * @param param
     * @return
     */
    @Override
    public R onlineCamera(CameraOnlineStatus param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.onlineCamera;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
        Map map = JSON.parseObject(result, Map.class);
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }

    /**
     * 获取编码设备在线状态
     *
     * @param param
     * @return
     */
    @Override
    public R onlineEncodeDevice(EncodeDeviceOnlineStatus param) {
        //STEP1：设置平台参数，根据实际情况,设置host appkey appsecret 三个参数.

        ArtemisConfig.host = cameraConfig.getHost(); // artemis网关服务器ip端口
        ArtemisConfig.appKey = cameraConfig.getAppKey();  // 秘钥appkey
        ArtemisConfig.appSecret = cameraConfig.getAppSecret();// 秘钥appSecret

        //STEP2：设置OpenAPI接口的上下文
        final String ARTEMIS_PATH = cameraConfig.getPath();
        String protocol = cameraConfig.getProtocol();

        //STEP3：设置接口的URI地址
        final String previewURLsApi = ARTEMIS_PATH + IscCameraConstants.onlineEncodeDevice;

        Map<String, String> path = new HashMap<String, String>(2) {
            {
                put(protocol + "://", previewURLsApi);//根据现场环境部署确认是http还是https
            }
        };

        //STEP4：设置参数提交方式
        String contentType = "application/json";

        //STEP5：组装请求参数
        String body = JSON.toJSONString(param);

        //STEP6：调用接口
        String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
        Map map = JSON.parseObject(result, Map.class);
        if (!CollectionUtils.isEmpty(map) && "0".equals(map.get("code"))) {
            return R.ok(map.get("data"));
        }
        return R.error((String) map.get("msg"));
    }
}
