package cn.wisenergy.service.util;

import java.awt.image.BufferedImage;
import java.io.*;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;


import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.Transparency;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.util.Base64;
import java.util.Date;
import java.util.Random;


public class ImageUtils {

    public static String getBase64ByImgUrl(String url) {
        String suffix = url.substring(url.lastIndexOf(".") + 1);
        try {
            URL urls = new URL(url);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            Image image = Toolkit.getDefaultToolkit().getImage(urls);
            BufferedImage biOut = toBufferedImage(image);
            ImageIO.write(biOut, suffix, baos);
            String base64Str = Base64Util.encode(baos.toByteArray());
            return base64Str;
        } catch (Exception e) {
            return "";
        }

    }

    public static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) {
            return (BufferedImage) image;
        }
        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment
                .getLocalGraphicsEnvironment();
        try {
            int transparency = Transparency.OPAQUE;
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(image.getWidth(null),
                    image.getHeight(null), transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }
        if (bimage == null) {
            // Create a buffered image using the default color model
            int type = BufferedImage.TYPE_INT_RGB;
            bimage = new BufferedImage(image.getWidth(null),
                    image.getHeight(null), type);
        }
        // Copy image to buffered image
        Graphics g = bimage.createGraphics();
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
        return bimage;
    }

    /**
     * 通过图片的url获取图片的base64字符串
     *
     * @param imgUrl 图片url
     * @return 返回图片base64的字符串
     */
    public static String image2Base64(String imgUrl) {

        URL url = null;

        InputStream is = null;

        ByteArrayOutputStream outStream = null;

        HttpURLConnection httpUrl = null;

        try {

            url = new URL(imgUrl);

            httpUrl = (HttpURLConnection) url.openConnection();

            httpUrl.connect();

            httpUrl.getInputStream();

            is = httpUrl.getInputStream();


            outStream = new ByteArrayOutputStream();

            //创建一个Buffer字符串

            byte[] buffer = new byte[1024];

            //每次读取的字符串长度，如果为-1，代表全部读取完毕

            int len = 0;

            //使用一个输入流从buffer里把数据读取出来

            while ((len = is.read(buffer)) != -1) {

                //用输出流往buffer里写入数据，中间参数代表从哪个位置开始读，len代表读取的长度

                outStream.write(buffer, 0, len);

            }

            // 对字节数组Base64编码

            return Base64Util.encode(outStream.toByteArray());

        } catch (Exception e) {

            e.printStackTrace();

        } finally {

            if (is != null) {

                try {

                    is.close();

                } catch (IOException e) {

                    e.printStackTrace();

                }

            }

            if (outStream != null) {

                try {

                    outStream.close();

                } catch (IOException e) {

                    e.printStackTrace();

                }

            }

            if (httpUrl != null) {

                httpUrl.disconnect();

            }

        }

        return imgUrl;

    }

    //先将base64转成字节数在
    public static byte[] base64ToByte(String imageBase64) {
        byte[] b = null;
        Base64.Decoder decoder = Base64.getDecoder();
        try {
            if (imageBase64.indexOf("data:image/jpeg;base64,") != -1) {
                b = decoder.decode(imageBase64.replaceAll("data:image/jpeg;base64,", ""));
            } else {
                if (imageBase64.indexOf("data:image/png;base64,") != -1) {
                    b = decoder.decode(imageBase64.replaceAll("data:image/png;base64,", ""));
                } else {
                    b = decoder.decode(imageBase64.replaceAll("data:image/jpg;base64,", ""));
                }
            }
            for (int i = 0; i < b.length; ++i) {
                if (b[i] < 0) {// 调整异常数据
                    b[i] += 256;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return b;
    }

    /**
     * {@code 功能描述: 将二进制转换为图片
     *
     * @param b        二进制
     * @param userName 用户名
     * @return 图片路径 如果是null 就是保存失败
     * @author hzj
     * @since 2019-7-23 17:53:21
     */
    public static String filePath(byte[] b) {
        String imgPath = null;   //最终返回的图片路径
        try {
            //为了避免重复，图片名字改成 用户名+当前时间+图片格式
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
            String fileAdd = sdf.format(new Date()); //当前时间
            //图片存储路径
            //如果要存到当前项目路径下就用这个
            //String resourcesPath = new UploaderUtil().getClass().getResource("/").getPath();
            // 自定义路径
            String resourcesPath = "C:\\image\\";
            //做转码,否则存的图片名字会乱码

            resourcesPath = URLDecoder.decode(resourcesPath, "UTF-8");
            //图片名称是  当前日期 + 随机数   避免数据重复
            String fileName = fileAdd + new Random().nextInt(1000) + ".jpg";
            //用于写入的图片路径
            String Path = resourcesPath + fileName;
            //如果自定义路径,首先判断路径是否存在
            File directory = new File(resourcesPath);
            if (!directory.exists()) {
                directory.mkdir();
            }
            //如果文件夹不存在则创建
            File file = new File(Path);
            if (!file.exists()) {
                file.createNewFile();
            }

            //将二进制在转换为图片
            OutputStream out = new FileOutputStream(Path);
            out.write(b);
            out.flush();
            out.close();
            imgPath = fileName;
        } catch (Exception e) {
            imgPath = null;
            e.printStackTrace();
        }
        return imgPath;
    }
}
