package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.app.Camera;
import cn.wisenergy.model.vo.CameraQueryVo;
import cn.wisenergy.service.app.CameraService;
import cn.wisenergy.service.app.ISiteWatchOverService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


/**
 * @ Description   :  摄像机控制层
 * @ Author        :  lcc
 * @ CreateDate    :  2020/11/3 16:45
 */
@RestController
@Api(tags = "系统设置--摄像机管理")
@RequestMapping(value = "/camera")
@Slf4j
public class CameraController {

    @Autowired
    private CameraService cameraService;

    @Autowired
    private ISiteWatchOverService siteWatchOverService;

    @ApiOperation(value = "添加摄像机", notes = "添加摄像机", httpMethod = "POST")
    @ApiImplicitParam(name = "camera", value = "摄像机信息", dataType = "Camera")
    @PostMapping("/add")
    public R add(@RequestBody Camera camera) {
        if (null == camera) {
            return R.error("操作错误");
        }
        return cameraService.add(camera);
    }


    @ApiOperation(value = "编辑摄像机", notes = "编辑摄像机", httpMethod = "POST")
    @ApiImplicitParam(name = "camera", value = "编辑摄像机信息", dataType = "Camera")
    @PostMapping("/edit")
    public R edit(@RequestBody Camera camera) {
        if (null == camera) {
            return R.error("操作错误");
        }
        return cameraService.edit(camera);
    }

    @ApiOperation(value = "删除摄像机", notes = "删除摄像机", httpMethod = "DELETE")
    @ApiImplicitParam(name = "id", value = "摄像机id", dataType = "int", required = true)
    @DeleteMapping("/delete")
    public R delete(Integer id) {
        if (null == id) {
            return R.error("操作错误");
        }
        return cameraService.delete(id);
    }

    @ApiOperation(value = "获取摄像机列表", notes = "获取摄像机列表", httpMethod = "GET")
    @ApiImplicitParam(name = "rubbishQueryVo", value = "条件参数", dataType = "RubbishQueryVo")
    @GetMapping("/getList")
    public R getList(CameraQueryVo cameraQueryVo) {
        if (null == cameraQueryVo) {
            return R.error("操作错误");
        }
        return cameraService.getList(cameraQueryVo);
    }

    @ApiOperation(value = "获取垃圾站名称列表", notes = "获取垃圾站名称列表", httpMethod = "GET")
    @GetMapping("/getRubbishNameList")
    public R getRubbishNameList() {
        return cameraService.getRubbishNameList();
    }

}
