package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.service.app.CommunityBrainService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @ Description   :  社区大脑
 * @ Author        :  lcc
 * @ CreateDate    :  2020/12/2 14:22
 */
@RestController
@Api(tags = "社区大脑")
@RequestMapping(value = "/camera")
@Slf4j
public class CommunityBrainController {

    @Autowired
    private CommunityBrainService communityBrainService;

    @ApiOperation(value = "统计详情", notes = "统计详情", httpMethod = "GET")
    @ApiImplicitParam(name = "autoId", value = "登录用户autoId", dataType = "String")
    @GetMapping("/getCountDetail")
    public R getCountDetail(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("操作错误");
        }
        return communityBrainService.getCountDetail(autoId);
    }

    @ApiOperation(value = "人员总数", notes = "人员总数", httpMethod = "GET")
    @ApiImplicitParam(name = "autoId", value = "登录用户autoId", dataType = "String")
    @GetMapping("/userRoleCount")
    public R userRoleCount(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("操作错误");
        }
        return communityBrainService.userRoleCount(autoId);
    }

    @ApiOperation(value = "社区大脑-小区/街道督查率排名", notes = "社区大脑-小区/街道督查率排名", httpMethod = "GET")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "autoId", value = "登录用户autoId", dataType = "String"),
            @ApiImplicitParam(name = "type", value = "1: 街道 2：社区", dataType = "int")
    })
    @GetMapping("/getInspectorRate")
    public R getInspectorRate(String autoId, Integer type) {
        if (StringUtils.isBlank(autoId) || null == type) {
            return R.error("操作错误");
        }
        return communityBrainService.getInspectorRate(autoId, type);
    }

    @ApiOperation(value = "社区大脑-违规派单", notes = "社区大脑-违规派单", httpMethod = "GET")
    @ApiImplicitParam(name = "autoId", value = "登录用户autoId", dataType = "String")
    @GetMapping("/getViolationOrder")
    public R getViolationOrder(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("操作错误");
        }
        return communityBrainService.getViolationOrder(autoId);
    }


    @ApiOperation(value = "社区大脑-总派单数/派单反馈数", notes = "社区大脑-总派单数/派单反馈数", httpMethod = "GET")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "autoId", value = "登录用户autoId", dataType = "String"),
            @ApiImplicitParam(name = "type", value = "1:今日 2：本周 3：本月 4：本季 5：本年", dataType = "int")
    })
    @GetMapping("/getSendOrderCount")
    public R getSendOrderCount(String autoId, Integer type) {
        if (StringUtils.isBlank(autoId) || null == type) {
            return R.error("操作错误");
        }
        return communityBrainService.getSendOrderCount(autoId, type);
    }

    @ApiOperation(value = "社区大脑-清运团队清运率/清运次数统计", notes = "社区大脑-清运团队清运率/清运次数统计", httpMethod = "GET")
    @ApiImplicitParam(name = "autoId", value = "登录用户autoId", dataType = "String")
    @GetMapping("/getClearTeam")
    public R getClearTeam(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("操作错误");
        }
        return communityBrainService.getClearTeam(autoId);
    }

}
