package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.app.Camera;
import cn.wisenergy.model.app.FfmpegBackRecord;
import cn.wisenergy.model.dto.CameraHistoryDto;
import cn.wisenergy.model.dto.CameraListDto;
import cn.wisenergy.model.dto.CameraLiveDto;
import cn.wisenergy.service.app.CameraService;
import cn.wisenergy.service.app.IFfmpegBackRecordService;
import cn.wisenergy.service.app.ISiteWatchOverService;
import cn.wisenergy.service.ffmpeg.hkws.CommandManager;
import cn.wisenergy.service.ffmpeg.hkws.DTO.ResultDTO;
import cn.wisenergy.service.ffmpeg.hkws.constants.ErrorCodeConsts;
import cn.wisenergy.service.ffmpeg.hkws.controller.web;
import cn.wisenergy.service.ffmpeg.hkws.data.CommandTasker;
import cn.wisenergy.service.ffmpeg.hkws.enumeration.ResultEnum;
import cn.wisenergy.service.ffmpeg.hkws.exception.GlobalException;
import cn.wisenergy.web.admin.controller.app.schedule.FfmpegScheduleComponent;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.List;
import java.util.Objects;

@RestController
@RequestMapping("ffmpeg")
@Slf4j
public class FfmpegController {

    public static CommandManager manager = web.manager;


    @Autowired
    CameraService cameraService;

    @Autowired
    IFfmpegBackRecordService ffmpegBackRecordService;

    @Autowired
    FfmpegScheduleComponent ffmpegScheduleComponent;

    @Autowired
    ISiteWatchOverService siteWatchOverService;


    @GetMapping("/test1")
    public String hello() {
//        ffmpegScheduleComponent.autoPushRtmp();
        return null;
    }


    @GetMapping("getCameraList")
    @ApiOperation(value = "获取摄像机列表")
    public ResultDTO getCameraList() {
        QueryWrapper<Camera> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_delete", 0)
                .in("");
        List<Camera> list = cameraService.list(queryWrapper);
        return ResultDTO.of(ResultEnum.SUCCESS, list);
    }


    @GetMapping("/cameraList/condition")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "autoId", value = "组织树autoId", dataType = "String"),
            @ApiImplicitParam(name = "userAutoId", value = "登录用户autoId", dataType = "String", required = true)
    })
    @ApiOperation(value = "通过社区查询垃圾站下的摄像头回放URLs，带分页")
    public R pageCameraBackUrls(@Validated CameraListDto cameraListDto) {
        if (StringUtils.isBlank(cameraListDto.getUserAutoId()) ||
                StringUtils.isBlank(cameraListDto.getAutoId())) {
            return R.error("入参为空！");
        }
        return siteWatchOverService.pageCameraBackUrls(cameraListDto);
    }


    @GetMapping("/getCameraList/condition")
    @ApiOperation(value = "获取定时任务已经推送的回放URL")
    public ResultDTO getCameraListCondition( CameraListDto cameraListDto) {
        //
        QueryWrapper<Camera> cameraQueryWrapper = new QueryWrapper<>();
        cameraQueryWrapper.in("id", cameraListDto.getIds())
                .eq("is_delete", 0);
        List<Camera> cameras = cameraService.list(cameraQueryWrapper);
        //
        QueryWrapper<FfmpegBackRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper
                .in("camear_id", cameraListDto.getIds())
                .gt("begin_time", cameraListDto.getBeginTime())
                .eq("status", 0)
                .lt("end_time", cameraListDto.getEndTime());
        List<FfmpegBackRecord> list = ffmpegBackRecordService.list(queryWrapper);
        //
        cameras.stream().forEach(c -> {
            list.stream().forEach(l -> {
                if (l.getCamearId() != null && l.getCamearId().equals(c.getId())) {
                    l.setCameraNumber(c.getCameraNumber());
                }
            });
        });
        return ResultDTO.of(ResultEnum.SUCCESS, list);
    }


    @PostMapping("/getLiveStream")
    @ApiOperation(value = "window,获取实时视频流", notes = "channelStream:101 代表通道一 主码流。102代表通道一 子码流；子码流比主码流小，但画质会有所下降 ，channelName是通道名，能保证唯一即可")
    public ResultDTO live(@RequestBody CameraLiveDto liveDTO) {

        //根据摄像机id查询摄像机
        Camera byId = cameraService.getById(liveDTO.getId());
        if (byId.getIp() == null) {
            return ResultDTO.of(ResultEnum.ERROR, "当前摄像机没有配置固定ip等信息");
        }
        String channelName = "live_" + liveDTO.getId();   //任务id  唯一即可
        String realPlayRtsp = realPlayRtsp(byId, 101);
        String childDir = channelName;
        String realPlayRtmp = realPlayRtmp("172.18.1.56", "1935", childDir);

        String liveUrl = "";

//        String channelName = liveDTO.getChannelName();


        //通过id查询这个任务
        CommandTasker info = manager.query(channelName);
//       如果任务没存在，开启视频流
        if (Objects.isNull(info)) {
//            manager.start(channelName, "ffmpeg -re  -rtsp_transport tcp -i \"rtsp://" + winAccountInfo + "/Stream/Channels/" + liveDTO.getChannelStream() + "\" -f flv -vcodec copy -acodec copy -s 1480*500 -crf 15 \"rtmp://172.18.1.56:1935/hls/" + channelName + "\"");

            manager.start(channelName, "ffmpeg -re  -rtsp_transport tcp -i" +
                    " \"" + realPlayRtsp + "\"" +
                    " -f flv -vcodec copy -acodec copy -s 1480*500 -crf 15 " +
                    "\"" + realPlayRtmp + "\"");

//            manager.start(channelName, "ffmpeg -re  -rtsp_transport tcp -i \"rtsp://"+winAccountInfo+"/Streaming/Channels/"+liveDTO.getChannelStream()+"?transportmode=unicast\" -f flv -vcodec copy -acodec copy -s 1480*500 -crf 15 \"rtmp://localhost:1935/live/\""+channelName);
        }
        // 如果是window rtmp版就返回 rtmp://localhost:1935/live/\""+channelName

//        liveUrl = "hls://172.18.1.56:1935/hls/" + channelName + "/index.m3u8";
        liveUrl = liveUrl(channelName);
        // 下面这个是http-flv版的流
//        liveUrl= "/live?port=1935&app=myapp&stream="+channelName;

        return ResultDTO.of(ResultEnum.SUCCESS).setData(liveUrl);
    }


    @PostMapping("/getHistoryStream")
    @ApiOperation(value = "windows 获取历史视频流", notes = "channelStream:101 通道一 码流一，channelName是通道名，能保证唯一即可,跟前端约定就以 history为前缀，避免跟上面的channelName冲突了")
    public ResultDTO history(@RequestBody CameraHistoryDto historyDTO) throws GlobalException {
        String liveUrl = "";

//        String channelName = historyDTO.getChannelName();
//        String starttime = timeStampString(historyDTO.getStarttime());
        String starttime = timeStampStringB(historyDTO.getStarttime());
//        String endtime = timeStampString(historyDTO.getEndtime());
        String endtime = timeStampStringE(historyDTO.getEndtime());
        Integer id = historyDTO.getId();

        Camera byId = cameraService.getById(id);
        if (byId.getIp() == null) {
            return ResultDTO.of(ResultEnum.ERROR, "当前摄像机没有配置固定ip等信息");
        }
        //先根据条件查询，有则直接返回
        FfmpegBackRecord one = ffmpegBackRecordService.getOneByCondition(id,
                historyDTO.getStarttime(), historyDTO.getEndtime());
        if (one != null) {
            return ResultDTO.of(ResultEnum.SUCCESS).setData(one.getBackPlayUrl());
        }

        String channelName = "back_" + id + ":" +
                starttime + ":" + endtime;   //唯一即可
        //拼接拉流URL
        String backPlayRtsp = backPlayRtsp(byId, 101, starttime, endtime);
        String backPlayRtmp = backPlayRtmp("172.18.1.56", "1935", channelName);

        //通过id查询这个任务
        CommandTasker info = manager.query(channelName);
//       如果任务没存在，开启视频流
        if (Objects.isNull(info)) {
            //执行原生ffmpeg命令（不包含ffmpeg的执行路径，该路径会从配置文件中自动读取）
            try {
                String result = manager.start(channelName, "ffmpeg -re  -rtsp_transport tcp -i " +
                        "\"" + backPlayRtsp + "\" " +
                        " -f flv -vcodec h264 -acodec aac -ar 44100  -s 1480*500 -crf 15 " +
                        "\"" + backPlayRtmp + "\"");

                log.info("result" + result);

            } catch (Exception e) {
                log.info("windows history:" + e.getMessage());

                throw new GlobalException(ErrorCodeConsts.ERROR, e.getMessage());
            }
        } else {
            log.info(channelName + "任务已存在，从任务列表中取出数据返回");
        }
        // 如果是window rtmp版就返回 rtmp://localhost:1935/live/\""+channelName
//        liveUrl = "hls://172.18.1.56:1935/hls/" + channelName + "/index.m3u8";
        liveUrl = backUrl(channelName);
        // 下面这个是http-flv版的流
//        liveUrl= "/live?port=1935&app=myapp&stream="+channelName;

        //更新数据表设置当前的摄像机回放记录
        FfmpegBackRecord ffmpegBackRecord = new FfmpegBackRecord();
        ffmpegBackRecord.setBackPlayUrl(liveUrl);
        ffmpegBackRecord.setBeginTime(historyDTO.getStarttime());
        ffmpegBackRecord.setEndTime(historyDTO.getEndtime());
        ffmpegBackRecord.setCamearId(byId.getId());
        ffmpegBackRecord.setChannelName(channelName);
        ffmpegBackRecord.setStatus(1);
        boolean save = ffmpegBackRecordService.save(ffmpegBackRecord);

        return ResultDTO.of(ResultEnum.SUCCESS).setData(liveUrl);
    }


    public static String timeStampString(Long timeStamp) {
        //20201220t120000z
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyyMMdd't'HHmmss'z'");
        //转化时间戳为  localdatetime
        LocalDateTime localDateTime = Instant.ofEpochMilli(timeStamp).atOffset(ZoneOffset.ofHours(8)).toLocalDateTime();
        String format = dateTimeFormatter.format(localDateTime);
//        int year = localDateTime.get(ChronoField.YEAR);
//        int month = localDateTime.get(ChronoField.MONTH_OF_YEAR);
//        int day = localDateTime.get(ChronoField.DAY_OF_MONTH);
//        int hour = localDateTime.get(ChronoField.HOUR_OF_DAY);
//        int minute = localDateTime.get(ChronoField.MINUTE_OF_HOUR);
//        int second = localDateTime.get(ChronoField.SECOND_OF_MINUTE);
//        StringBuilder stringBuilder = new StringBuilder();
//
//        StringBuilder builder = stringBuilder.append(year).append(month).append(day).append("t")
//                .append(hour).append(minute).append(second);
//
//        String string = builder.toString();

        return format;
    }


    public static String timeStampStringB(Long timeStamp) {
        //20201220t120000z
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyyMMdd't'HHmmss'z'");
        //转化时间戳为  localdatetime
        LocalDateTime localDateTime = Instant.ofEpochMilli(timeStamp).minus(8, ChronoUnit.HOURS)
                .atOffset(ZoneOffset.ofHours(8)).toLocalDateTime();
        String format = dateTimeFormatter.format(localDateTime);


        return format;
    }

    public static String timeStampStringE(Long timeStamp) {
        //20201220t120000z
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyyMMdd't'HHmmss'z'");
        //转化时间戳为  localdatetime
        LocalDateTime localDateTime = Instant.ofEpochMilli(timeStamp).minus(8, ChronoUnit.HOURS)
                .atOffset(ZoneOffset.ofHours(8)).toLocalDateTime();
        String format = dateTimeFormatter.format(localDateTime);
//        int year = localDateTime.get(ChronoField.YEAR);
//        int month = localDateTime.get(ChronoField.MONTH_OF_YEAR);
//        int day = localDateTime.get(ChronoField.DAY_OF_MONTH);
//        int hour = localDateTime.get(ChronoField.HOUR_OF_DAY);
//        int minute = localDateTime.get(ChronoField.MINUTE_OF_HOUR);
//        int second = localDateTime.get(ChronoField.SECOND_OF_MINUTE);
//        StringBuilder stringBuilder = new StringBuilder();
//
//        StringBuilder builder = stringBuilder.append(year).append(month).append(day).append("t")
//                .append(hour).append(minute).append(second);
//
//        String string = builder.toString();

        return format;
    }

    /**
     * 根据摄像机回放   拼接拉流URL
     *
     * @param camera
     * @return
     */
    public static String backPlayRtsp(Camera camera, Integer channel, String starttime, String endtime) {
        //rtsp://admin:Dapeng123456@192.168.0.177:554/Streaming/tracks/101?starttime=20201220t093000z&endtime=20201220t120000z
        StringBuilder stringBuilder = new StringBuilder();
        StringBuilder append = stringBuilder.append("rtsp://")
                .append(camera.getUsername())
                .append(":").append(camera.getPassword())
                .append("@").append(camera.getIp()).append(":").append("554")
                .append("/Streaming/tracks/").append(channel)
                .append("?").append("starttime=").append(starttime).append("z")
                .append("&").append("endtime=").append(endtime);
        String string = append.toString();
        return string;
    }


    /**
     * 根据摄像机实时直播   拼接拉流URL
     *
     * @param camera  摄像机
     * @param channel 通道
     * @return
     */
    public static String realPlayRtsp(Camera camera, Integer channel) {
        //rtsp://admin1:Dapeng12345678@192.168.0.177:554/Stream/Channels/101

        StringBuilder stringBuilder = new StringBuilder();
        StringBuilder append = stringBuilder.append("rtsp://")
                .append(camera.getUsername())
                .append(":").append(camera.getPassword())
                .append("@").append(camera.getIp()).append(":").append("554")
                .append("/Stream/Channels/").append(channel);
        String string = append.toString();
        return string;
    }


    /**
     * 直播推送  rtmp  ffmepg 服务器地址
     *
     * @return
     */
    public static String realPlayRtmp(String ip, String port, String childDir) {
        //rtmp://172.18.1.56:1935/hls/
        StringBuilder stringBuilder = new StringBuilder();

        StringBuilder append = stringBuilder.append("rtmp://")
                .append(ip).append(":").append(port)
                .append("/hls/")
                .append(childDir);
        String string = append.toString();
        return string;
    }


    /**
     * 回放推送  rtmp  ffmepg 服务器地址
     *
     * @return
     */
    public static String backPlayRtmp(String ip, String port, String childDir) {
        //rtmp://172.18.1.56:1935/hls/
        StringBuilder stringBuilder = new StringBuilder();

        StringBuilder append = stringBuilder.append("rtmp://")
                .append(ip).append(":").append(port)
                .append("/vod/")
                .append(childDir);
        String string = append.toString();
        return string;
    }


    /**
     * 直播URL访问地址
     *
     * @return
     */
    public static String liveUrl(String childDir) {
        //"hls://172.18.1.56:1935/hls/" + channelName + "/index.m3u8";
        ////  http://111.203.232.172改成  https://www.duchabao.top

        StringBuilder stringBuilder = new StringBuilder();

        StringBuilder append = stringBuilder.append("https://")
                .append("www.duchabao.top").append(":").append(8080)
                .append("/hls")
                .append("/").append(childDir)
                .append("/").append("index.m3u8");
        String string = append.toString();
        return string;
    }


    /**
     * 回放URL访问地址
     *
     * @return
     */
    public static String backUrl(String childDir) {
        //"hls://172.18.1.56:1935/hls/" + channelName + "/index.m3u8";
        //  http://111.203.232.172改成  https://www.duchabao.top
        StringBuilder stringBuilder = new StringBuilder();

        StringBuilder append = stringBuilder.append("https://")
                .append("www.duchabao.top").append(":").append(8080)
                .append("/vod")
                .append("/").append(childDir)
                .append("/").append("index.m3u8");
        String string = append.toString();
        return string;
    }
}
