package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.app.NewContent;
import cn.wisenergy.model.vo.*;
import cn.wisenergy.service.app.NewContentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

/**
 * @ Description   :  摄像机控制层
 * @ Author        :  lcc
 * @ CreateDate    :  2020/11/30 16:45
 */
@RestController
@Api(tags = "系统设置--新闻、手册管理")
@RequestMapping(value = "/content")
@Slf4j
public class NewContentController {

    @Autowired
    private NewContentService newContentService;


    @ApiOperation(value = "添加新闻/手册", notes = "添加新闻/手册", httpMethod = "POST")
    @ApiImplicitParam(name = "newContent", value = "新闻/手册信息", dataType = "NewContent")
    @PostMapping("/add")
    public R add(@RequestBody NewContent newContent) {
        if (null == newContent) {
            return R.error("操作错误");
        }
        AccessToken token = newContentService.getToken();
        if (StringUtils.isNotEmpty(token.getErrcode())) {
            return R.error("token无效!");
        }
        return newContentService.add(newContent, token.getAccess_token());
    }


    @ApiOperation(value = "编辑新闻/手册", notes = "编辑新闻/手册", httpMethod = "POST")
    @ApiImplicitParam(name = "newContent", value = "编辑新闻/手册信息", dataType = "NewContent")
    @PostMapping("/edit")
    public R edit(@RequestBody NewContent newContent) {
        if (null == newContent) {
            return R.error("操作错误");
        }
        AccessToken token = newContentService.getToken();
        if (StringUtils.isNotEmpty(token.getErrcode())) {
            return R.error("token无效!");
        }
        return newContentService.edit(newContent, token.getAccess_token());
    }

    @ApiOperation(value = "删除新闻/手册", notes = "删除新闻/手册", httpMethod = "DELETE")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "新闻/手册id", dataType = "int", required = true),
            @ApiImplicitParam(name = "type", value = "类型：1：新闻   2：手册", dataType = "int", required = true)
    })
    @DeleteMapping("/delete")
    public R delete(Integer id, Integer type) {
        if (null == id || null == type) {
            return R.error("操作错误");
        }
        return newContentService.delete(id, type);
    }

    @ApiOperation(value = "获取新闻/手册列表", notes = "获取新闻/手册列表", httpMethod = "GET")
    @ApiImplicitParam(name = "queryVo", value = "条件参数", dataType = "NewContentQueryVo")
    @GetMapping("/getList")
    public R getList(NewContentQueryVo queryVo) {
        if (null == queryVo) {
            return R.error("操作错误");
        }
        return newContentService.getList(queryVo);
    }

    @ApiOperation(value = "获取token", notes = "获取token", httpMethod = "GET")
    @GetMapping("/getToken")
    public Object getToken() {
        log.info("");
        return newContentService.getToken();
    }

    @ApiOperation(value = "图文消息留言管理", notes = "图文消息留言管理", httpMethod = "POST")
    @ApiImplicitParam(name = "messageVo", value = "图文消息", dataType = "NewMessageVo")
    @PostMapping("/addNew")
    public Object addNew(@RequestBody NewMessageVo messageVo) {
        if (null == messageVo) {
            return R.error("操作错误");
        }
        AccessToken token = newContentService.getToken();
        if (StringUtils.isNotEmpty(token.getErrcode())) {
            return R.error("token无效!");
        }
        return newContentService.addNew(messageVo, token.getAccess_token());
    }

    @ApiOperation(value = "修改图文消息", notes = "修改图文消息", httpMethod = "POST")
    @ApiImplicitParam(name = "messageVo", value = "图文消息", dataType = "NewMessageVo")
    @PostMapping("/updateNew")
    public Object updateNew(@RequestBody NewMessageVo messageVo) {
        if (null == messageVo) {
            return R.error("操作错误");
        }
        AccessToken token = newContentService.getToken();
        if (StringUtils.isNotEmpty(token.getErrcode())) {
            return R.error("token无效!");
        }
        return newContentService.updateMaterial(messageVo, token.getAccess_token());
    }

    @ApiOperation(value = "获取素材信息", notes = "获取素材信息", httpMethod = "POST")
    @ApiImplicitParam(name = "requestMaterialVo", value = "素材media_id", dataType = "RequestMaterialVo")
    @PostMapping("/getMaterial")
    public Object getMaterial(@RequestBody RequestMaterialVo  requestMaterialVo) {
        if (null == requestMaterialVo) {
            return R.error("操作错误");
        }
        AccessToken token = newContentService.getToken();
        if (StringUtils.isNotEmpty(token.getErrcode())) {
            return R.error("token无效!");
        }
        return newContentService.getMaterial(requestMaterialVo, token.getAccess_token());
    }

    @ApiOperation(value = "上传媒体素材", notes = "上传媒体素材", httpMethod = "POST")
    @PostMapping(value = "/uploadMaterial/{type}", headers = "content-type=multipart/form-data")
    public R<UploadMaterialVo> uploadMaterial(@RequestParam("file") MultipartFile file,
                                              @PathVariable("type") String type,
                                              HttpServletRequest request) throws UnsupportedEncodingException {
        if (StringUtils.isBlank(type) || null == file) {
            return R.error("入参为空!");
        }
        AccessToken token = newContentService.getToken();
        if (StringUtils.isNotEmpty(token.getErrcode())) {
            return R.error("token无效!");
        }

        try {
            String fileName = file.getOriginalFilename();
            fileName = URLEncoder.encode(fileName, "UTF-8");// 进行中文处理
            String UPLOAD_PATH = File.separator + "upload" + File.separator + "img" + File.separator;
            String path = request.getSession().getServletContext().getRealPath(UPLOAD_PATH);
            String pathFileName = path + File.separator + fileName;
            file.transferTo(new File(pathFileName));
            return newContentService.addMaterialEver(pathFileName, type, token.getAccess_token());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return R.error("上传" + type + "失败!");
    }

    @ApiOperation(value = "获取素材列表", notes = "获取素材列表", httpMethod = "POST")
    @ApiImplicitParam(name = "requestMaterialVo", value = "参数", dataType = "WxDto")
    @PostMapping("/list")
    public Object list(@RequestBody WxDto  wxDto) {
        if (null == wxDto) {
            return R.error("操作错误");
        }
        AccessToken token = newContentService.getToken();
        if (StringUtils.isNotEmpty(token.getErrcode())) {
            return R.error("token无效!");
        }
        return newContentService.list(wxDto, token.getAccess_token());
    }
}
