package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.expection.BaseException;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.app.Resident;
import cn.wisenergy.model.vo.ResidentQueryVo;
import cn.wisenergy.service.app.ResidentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @ Description   :  居民控制层
 * @ Author        :  lcc
 * @ CreateDate    :  2020/11/3 16:45
 */
@RestController
@Api(tags = "系统设置--居民管理")
@RequestMapping(value = "/resident")
@Slf4j
public class ResidentController {

    @Autowired
    private ResidentService residentService;

    @ApiOperation(value = "添加居民", notes = "添加居民", httpMethod = "POST")
    @ApiImplicitParam(name = "residents", value = "居民信息", dataType = "Resident")
    @PostMapping("/add")
    public R add(@RequestBody Resident resident) {
        if (null == resident) {
            return R.error("操作错误");
        }
        return residentService.add(resident);
    }


    @ApiOperation(value = "编辑居民", notes = "编辑居民", httpMethod = "POST")
    @ApiImplicitParam(name = "resident", value = "编辑居民信息", dataType = "Resident")
    @PostMapping("/edit")
    public R edit(@RequestBody Resident resident) {
        if (null == resident) {
            return R.error("操作错误");
        }
        return residentService.edit(resident);
    }

    @ApiOperation(value = "删除居民", notes = "删除居民", httpMethod = "DELETE")
    @ApiImplicitParam(name = "id", value = "居民id", dataType = "int", required = true)
    @DeleteMapping("/delete")
    public R delete(Integer id) {
        if (null == id) {
            return R.error("操作错误");
        }
        return residentService.delete(id);
    }

    @ApiOperation(value = "获取居民列表", notes = "获取居民列表", httpMethod = "GET")
    @ApiImplicitParam(name = "rubbishQueryVo", value = "条件参数", dataType = "RubbishQueryVo")
    @GetMapping("/getList")
    public R getList(ResidentQueryVo residentQueryVo) {
        if (null == residentQueryVo) {
            return R.error("操作错误");
        }
        return residentService.getList(residentQueryVo);
    }

    @ApiOperation(value = "Excel批量添加社区居民", notes = "Excel批量添加社区居民")
    @ApiImplicitParam(name = "file", value = "excel文件", dataType = "MultipartFile")
    @PostMapping("/excel/add")
    public R excelAdd(@RequestBody MultipartFile file, HttpServletResponse response) throws IOException {
        if (file == null || file.isEmpty()) {
            throw new BaseException("操作错误");
        }
        residentService.excelAdd(file, response);
        return R.ok("添加成功");
    }

    @ApiOperation(value = "社区居民Excel模板", notes = "社区居民Excel模板")
    @PostMapping("/excel/template")
    public R excelTemplate(HttpServletResponse response) throws IOException {

        residentService.excelTemplate(response);
        return R.ok(response);
    }
}
