package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.app.Rubbish;
import cn.wisenergy.model.vo.RubbishQueryVo;
import cn.wisenergy.service.app.RubbishService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @ Description   :  垃圾站controller
 * @ Author        :  lcc
 * @ CreateDate    :  2020/11/3 16:34
 */
@RestController
@Api(tags = "系统设置--垃圾站管理")
@RequestMapping(value = "/rubbish")
@Slf4j
public class RubbishController  {
    @Autowired
    private RubbishService rubbishService;


    @ApiOperation(value = "添加垃圾站", notes = "添加垃圾站", httpMethod = "POST")
    @ApiImplicitParam(name = "rubbish", value = "垃圾站信息", dataType = "Rubbish")
    @PostMapping("/add")
    public R add(@RequestBody Rubbish rubbish) {
        if (null == rubbish) {
            return R.error("操作错误");
        }
        return rubbishService.add(rubbish);
    }


    @ApiOperation(value = "编辑垃圾站", notes = "编辑垃圾站", httpMethod = "POST")
    @ApiImplicitParam(name = "rubbish", value = "垃圾站信息", dataType = "Rubbish")
    @PostMapping("/edit")
    public R edit(@RequestBody Rubbish rubbish) {
        if (null == rubbish) {
            return R.error("操作错误");
        }
        return rubbishService.edit(rubbish);
    }

    @ApiOperation(value = "删除垃圾站", notes = "删除垃圾站", httpMethod = "DELETE")
    @ApiImplicitParam(name = "id", value = "垃圾站id", dataType = "int", required = true)
    @DeleteMapping("/delete")
    public R delete(Integer id) {
        if (null == id) {
            return R.error("操作错误");
        }
        return rubbishService.delete(id);
    }

    @ApiOperation(value = "获取垃圾站列表", notes = "获取垃圾站列表", httpMethod = "GET")
    @ApiImplicitParam(name = "rubbishQueryVo", value = "条件参数", dataType = "RubbishQueryVo")
    @GetMapping("/getList")
    public R getList(RubbishQueryVo rubbishQueryVo) {
        if (null == rubbishQueryVo) {
            return R.error("操作错误");
        }
        return rubbishService.getList(rubbishQueryVo);
    }
}
