package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.vo.OrderQueryVo;
import cn.wisenergy.model.vo.ReceiveOrderVo;
import cn.wisenergy.model.vo.ReceiveQuestVo;
import cn.wisenergy.model.vo.SendQuestVo;
import cn.wisenergy.service.app.SendOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


@RestController
@Api(tags = "派单-接单-管理")
@RequestMapping("/order")
@Slf4j
public class SendOrderController {

    @Autowired
    private SendOrderService sendOrderService;

    @ApiOperation(value = "派单", notes = "派单", httpMethod = "POST")
    @ApiImplicitParam(name = "sendQuestVo", value = "派单信息", dataType = "SendQuestVo")
    @PostMapping("/sendOrder")
    public R sendOrder(@RequestBody SendQuestVo sendQuestVo) {
        if (null == sendQuestVo) {
            return R.error("参数错误");
        }
        return sendOrderService.sendOrder(sendQuestVo);
    }

    @ApiOperation(value = "接单", notes = "接单", httpMethod = "POST")
    @ApiImplicitParam(name = "receiveOrderVo", value = "请求参数", dataType = "ReceiveOrderVo")
    @PostMapping("/receiveOrder")
    public R receiveOrder(@RequestBody ReceiveOrderVo receiveOrderVo) {
        if (null == receiveOrderVo) {
            return R.error("参数错误");
        }
        return sendOrderService.receiveOrder(receiveOrderVo);
    }

    @ApiOperation(value = "获取派单列表", notes = "获取派单列表", httpMethod = "GET")
    @ApiImplicitParam(name = "receiveOrderVo", value = "请求参数", dataType = "OrderQueryVo")
    @GetMapping("/getPage")
    public R getPage(OrderQueryVo orderQueryVo) {
        if (null == orderQueryVo) {
            return R.error("参数错误");
        }
        return sendOrderService.getPage(orderQueryVo);
    }

    @ApiOperation(value = "获取派单详情", notes = "获取派单详情", httpMethod = "GET")
    @ApiImplicitParam(name = "autoId", value = "派单AutoId", dataType = "String")
    @GetMapping("/getDetail")
    public R getDetail(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("参数错误");
        }
        return sendOrderService.getDetail(autoId);
    }


    @ApiOperation(value = "派单反馈", notes = "派单反馈", httpMethod = "GET")
    @ApiImplicitParam(name = "autoId", value = "派单autoId", dataType = "String")
    @GetMapping("/orderBlack")
    public R orderBlack(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("参数错误");
        }
        return sendOrderService.orderBlack(autoId);
    }

    @ApiOperation(value = "接单详情", notes = "接单详情", httpMethod = "GET")
    @ApiImplicitParam(name = "autoId", value = "派单autoId", dataType = "String")
    @GetMapping("/getReceiveDetail")
    public R getReceiveDetail(String autoId) {
        if (StringUtils.isBlank(autoId)) {
            return R.error("参数错误");
        }
        return sendOrderService.getReceiveDetail(autoId);
    }

    @ApiOperation(value = "获取接单列表", notes = "获取接单列表", httpMethod = "GET")
    //@ApiImplicitParam(name = "receiveQuestVo", value = "条件参数", dataType = "ReceiveQuestVo")
    @GetMapping("/getReceiveList")
    public R getReceiveList(ReceiveQuestVo receiveQuestVo) {
        if (null == receiveQuestVo) {
            return R.error("参数错误");
        }
        return sendOrderService.getList(receiveQuestVo);
    }
}
