package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.dto.*;
import cn.wisenergy.service.app.ISiteWatchOverService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 站点值守 相关接口
 *
 * @author jiawei
 */
@RestController
@RequestMapping("/site/watchOver")
@Api(tags = "站点值守")
@Slf4j
public class SiteWatchOverController {

    @Autowired
    ISiteWatchOverService siteWatchOverService;

    @GetMapping("/search/playDuration")
    @ApiOperation(value = "查询站点值守视频播放设置的时长")
    public R playDuration() {

        return null;
    }


    @GetMapping("/search/street")
    @ApiOperation(value = "查询街道列表")
    public R street() {

        return null;
    }

    @GetMapping("/search/apartment")
    @ApiOperation(value = "通过街道查询小区")
    public R apartment() {

        return null;
    }

    @GetMapping("/search/rubbishStation")
    @ApiOperation(value = "通过小区查询垃圾站")
    public R rubbishStation() {

        return null;
    }

    @GetMapping("/search/camera/code")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "autoId", value = "组织树autoId", dataType = "String"),
            @ApiImplicitParam(name = "userAutoId", value = "登录用户autoId", dataType = "String", required = true)
    })
    @ApiOperation(value = "通过社区查询垃圾站下的摄像头code")
    public R rubbishStation(String autoId, String userAutoId) {
        if (StringUtils.isBlank(userAutoId) || StringUtils.isBlank(autoId)) {
            return R.error("入参为空！");
        }

        return siteWatchOverService.getByAutoId(autoId, userAutoId);
    }

//=================================根据监控点操作摄像头========================================

    @GetMapping("/search/cameraIndex")
    @ApiOperation(value = "查询视频监控点", notes = "根据垃圾站或者小区查询所拥有的监控点")
    public R pageVideo(VideoPageQry qry) {
        R result = siteWatchOverService.pageVideo(qry);
        return result;
    }

    @GetMapping("/search/videoUrls")
    @ApiOperation(value = "根据多个监控点查询视频预览Urls", notes = "不推荐调用这个接口")
    public R videoUrls(CamerasPreviewUrls previewUrls) {
        R result = siteWatchOverService.videoUrls(previewUrls);
        return result;
    }

    @GetMapping("/search/videoUrl")
    @ApiOperation(value = "根据一个监控点查询视频预览Urls")
    public R videoUrl(CameraPreviewUrls previewUrls) {
        R result = siteWatchOverService.videoUrl(previewUrls);
        return result;
    }

    @GetMapping("/search/playbackUrl")
    @ApiOperation(value = "根据一个监控点查询录像回放Urls")
    public R playbackUrl(CameraPlaybackURLs playbackURL) {
        R result = siteWatchOverService.playbackUrl(playbackURL);
        return result;
    }

    @GetMapping("/search/playbackUrls")
    @ApiOperation(value = "根据多个监控点查询录像回放Urls", notes = "不推荐调用这个接口")
    public R playbackUrls(CameraPlaybacksURLs playbackURLs) {
        R result = siteWatchOverService.playbackUrls(playbackURLs);
        return result;
    }

    @PostMapping("/manual/capture")
    @ApiOperation(value = "根据一个监控点进行手动抓图")
    public R manualCapture(@RequestBody  CameraIndexCode cameraIndexCode) {
        if (null == cameraIndexCode) {
            return R.error("入参为空!");
        }

        log.info("根据一个监控点进行手动抓图,入参：" + cameraIndexCode);
        R result = siteWatchOverService.manualCapture(cameraIndexCode);
        log.info("根据一个监控点进行手动抓图,结果：" + result);
        return result;
    }

    @PutMapping("/record/lock")
    @ApiOperation(value = "根据一个监控点进行录像锁定与解锁")
    public R recordLock(@Valid CameraRecordLock recordLock) {
        R result = siteWatchOverService.recordLock(recordLock);
        return result;
    }

    @PostMapping("/record/start")
    @ApiOperation(value = "根据一个监控点进行手动录像开始操作")
    public R recordStart(@Valid CameraManualRecordStart recordStart) {
        R result = siteWatchOverService.recordStart(recordStart);
        return result;
    }

    @PostMapping("/record/stop")
    @ApiOperation(value = "根据一个监控点进行手动录像停止操作")
    public R recordStop(@Valid CameraManualRecordStop recordStop) {
        R result = siteWatchOverService.recordStop(recordStop);
        return result;
    }

    @GetMapping("/record/status")
    @ApiOperation(value = "根据一个监控点进行手动录像状态获取操作")
    public R recordStatus(@Valid CameraManualRecordStop recordStatus) {
        R result = siteWatchOverService.recordStatus(recordStatus);
        return result;
    }


    @GetMapping("/record/search")
    @ApiOperation(value = "根据一个监控点查询手动录像编号")
    public R recordSearch(@Valid CameraIndexCodes recordSearch) {
        R result = siteWatchOverService.recordSearch(recordSearch);
        return result;
    }


    @GetMapping("/indexCode/online")
    @ApiOperation(value = "根据条件分页参数获取监控点在线状态列表")
    public R onlineStatus(@Valid CameraOnlineStatus onlineStatus) {
        R result = siteWatchOverService.onlineStatus(onlineStatus);
        return result;
    }


    @ApiOperation(value = "分页获取安防平台监控点列表", notes = "分页获取安防平台监控点列表", httpMethod = "GET")
    //@ApiImplicitParam(name = "receiveQuestVo", value = "条件参数", dataType = "ReceiveQuestVo")
    @GetMapping("/pageCameraIndex")
    public R pageCameraIndex(CameraPage cameraPage) {

        return siteWatchOverService.pageCameraIndex(cameraPage);
    }
}
