package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.vo.StreetQueryVo;
import cn.wisenergy.service.app.StreetService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @ Description   :  街道管理控制类
 * @ Author        :  lcc
 * @ CreateDate    :  2020/11/4 14:39
 */
@RestController
@Api(tags = "系统设置--街道管理")
@RequestMapping("/street")
@Slf4j
public class StreetController {
    @Autowired
    private StreetService streetService;

    @ApiOperation(value = "获取街道列表", notes = "获取街道列表", httpMethod = "GET")
    @ApiImplicitParam(name = "streetQueryVo", value = "条件参数", dataType = "StreetQueryVo")
    @GetMapping("/getList")
    public R getList(StreetQueryVo streetQueryVo) {
        if (null == streetQueryVo) {
            return R.error("操作错误");
        }
        return streetService.getList(streetQueryVo);
    }

    @ApiOperation(value = "获取街道树结构", notes = "获取街道树结构", httpMethod = "GET")
    @GetMapping("/getTree")
    public R getTree() {
        log.info("ljfl-service[]StreetController[]getTree[]input.method");
        return streetService.getTree();
    }

    @ApiOperation(value = "获取所有街道信息", notes = "获取所有街道信息", httpMethod = "GET")
    @GetMapping("/getStreet")
    public R getStreet() {
        log.info("ljfl-service[]StreetController[]getStreet[]input.method");
        return streetService.getAllStreet();
    }

    @ApiOperation(value = "获取街道下社区信息", notes = "获取街道下社区信息", httpMethod = "GET")
    @ApiImplicitParam(name = "autoId", value = "街道autoId", dataType = "String")
    @GetMapping("/getByStreetId")
    public R getByStreetId(String autoId) {
        log.info("ljfl-service[]StreetController[]getTree[]input.method");
        return streetService.getByStreetId(autoId);
    }

    @ApiOperation(value = "获取当前登录用户区/县、街道、社区信息", notes = "获取当前登录用户区/县、街道、社区信息", httpMethod = "GET")
    @ApiImplicitParam(name = "autoId", value = "当前登录用户autoId", dataType = "String")
    @GetMapping("/getUserTree")
    public R getUserTree(String autoId) {
        log.info("ljfl-service[]StreetController[]getTree[]input.method");
        return streetService.getUserTree(autoId);
    }
}
