package cn.wisenergy.web.admin.controller.app.schedule;

import cn.wisenergy.model.app.FfmpegBackRecord;
import cn.wisenergy.service.app.IFfmpegBackRecordService;
import cn.wisenergy.service.app.impl.FfmpegBackRecordServiceImpl;
import cn.wisenergy.service.ffmpeg.hkws.CommandManager;
import cn.wisenergy.service.ffmpeg.hkws.controller.web;
import cn.wisenergy.service.ffmpeg.hkws.util.ExecUtil;
import cn.wisenergy.web.admin.controller.app.FfmpegController;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.*;
import java.time.Instant;
import java.time.temporal.ChronoUnit;

import static cn.wisenergy.web.admin.controller.app.FfmpegController.backPlayRtmp;

/**
 * @author
 * @Date
 */
@Slf4j
@Component
@EnableScheduling
public class FfmpegScheduleComponent {

    @Autowired
    CameraHolder cameraHolder;

    public static CommandManager manager = web.manager;

    @Autowired
    IFfmpegBackRecordService ffmpegBackRecordService;

    /**
     * 每一小时执行一次
     */
    @Scheduled(cron = "0  0 0/1 * * ?")
    public void autoPushRtmp() {

        Instant realNow = Instant.now();  //当前时间
        Instant now = realNow.minus(8, ChronoUnit.HOURS);  //一小时前
        Instant oneBeforeHour = realNow.minus(9, ChronoUnit.HOURS);  //一小时前

        if (!cameraHolder.isLoad()) {
            cameraHolder.init();
            log.info(CameraHolder.class.getName() + ":  初始化属性值");
        }
        //调整 每次查询数据
        cameraHolder.init();
        //
        CameraHolder.cameras.forEach(c -> {
            Integer id = c.getId();
            //
            String starttime = FfmpegController.timeStampString(oneBeforeHour.toEpochMilli());
            String endtime = FfmpegController.timeStampString(now.toEpochMilli());

            String channelName = "back_" + id + ":" +
                    starttime + ":" + endtime;   //唯一即可
            //拼接拉流URL
            String backPlayRtsp = FfmpegController.backPlayRtsp(c, 101, starttime, endtime);
            String backPlayRtmp = backPlayRtmp("172.18.1.56", "1935", channelName);
            //
            String command = "ffmpeg -re  -rtsp_transport tcp -i" +
                    " \"" + backPlayRtsp + "\"" +
                    " -f flv -vcodec copy -acodec copy -s 1480*500 -crf 15 " +
                    "\"" + backPlayRtmp + "\"";

            String backUrl = FfmpegController.backUrl(channelName);

            FfmpegBackRecord ffmpegBackRecord = new FfmpegBackRecord();
            ffmpegBackRecord.setCamearId(c.getId());
            ffmpegBackRecord.setBackPlayUrl(backUrl);
            ffmpegBackRecord.setBeginTime(oneBeforeHour.toEpochMilli());
            ffmpegBackRecord.setEndTime(now.toEpochMilli());
            ffmpegBackRecord.setChannelName(channelName);
            ffmpegBackRecord.setStatus(1);   //默认就是不成功，认为本次会推送失败
            ffmpegBackRecordService.save(ffmpegBackRecord);

            //启动命令执行
            manager.start(channelName, command);

        });
    }


}
